@@ -54,7 +54,7 @@
 #
 # These variables control the setup
 #
-set addrVersion {$Revision: 1.1 $}
+set addrVersion {$Revision: 1.10 $}
 
 # Addr_debug, if ==1, enables printing messages while this stuff runs.
 # Set Addr_debug 1 in your user.tcl before invoking Addr_Init to enable debug messages
@@ -73,10 +73,10 @@ proc Addr_Init {} {
     global addr_list
 
     # addrFile is the name of the file the address database is kept in
-    set addrFile ".exmh_addrs"
+    set addrFile "exmh_addrs"
     
     # homeDir is the directory the database is kept in.
-    set homeDir $env(HOME)
+    set homeDir "$env(HOME)/.exmh"
 
     # tell exmh about our preference items and initialize them
     Preferences_Add "Address Database" "These settings affect the behavior of the address database.
@@ -106,9 +106,23 @@ proc Addr_Init {} {
         {
             addr_db(searchlist)
             addressdbSearchlist
-            {Addr_FullNameMatch Addr_Lookup Alias_Lookup}
+            {Addr_FullNameMatch Addr_Lookup Alias_Lookup LDAP_Lookup}
             "Expand methods to use"
-            "A list of TCL procs, separated by spaces, which will be called sequentially to try to expand the address.  Valid choices are \"Addr_FullNameMatch\" to search full names, \"Addr_Lookup\" to search for mail addresses, and \"Alias_Lookup\" to search your MH/exmh alias list."
+            "A list of TCL procs, separated by spaces, which will be called sequentially to try to expand the address.  Valid choices are \"Addr_FullNameMatch\" to search full names, \"Addr_Lookup\" to search for mail addresses, \"Alias_Lookup\" to search your MH/exmh alias list, and LDAP_Lookup to use an LDAP server."
+        }
+        {
+            addr_db(ldap_server)
+            addressdbLDAPServer
+            {}
+            "LDAP Server"
+            "The server to send LDAP queries to."
+        }
+        {
+            addr_db(ldap_searchbase)
+            addressdbLDAPSearchBase
+            {}
+            "LDAP Search Root"
+            "The root under which to conduct LDAP searches."
         }
         {
             addr_db(filter_regexp)
@@ -145,6 +159,27 @@ proc Addr_Init {} {
             "Use \"address (Full Name)\" Format"
             "If on, use \"address (Full Name)\" format for expanded addresses.  Otherwise, use \"Full Name <address>\" format."
         }
+        {
+            addr_db(remove_entries)
+            addressdbRemoveEntries
+            OFF
+            "Remove Old Entries"
+            "If on, remove old entries from the database"
+        }
+        {
+            addr_db(remove_days)
+            addressdbRemoveDays
+            {}
+            "Days Until Removal"
+            "Number of days until inactive entry is removed"
+        }
+        {
+            addr_db(remove_invalid_date)
+            addressdbRemoveInvalidDate
+            OFF
+            "Remove Invalid Date"
+            "If on, delete any entry with a non-null, but invalid date"
+        }
     }
 
     #addr_db is an array used for keeping state.
@@ -156,6 +191,9 @@ proc Addr_Init {} {
     set addr_db(lastfound) ""
     set addr_db(changed) 0
     set addr_db(filterstring) ""
+    set addr_db(remove_entries) 0
+    set addr_db(remove_days) ""
+    set addr_db(remove_invalid_date) 0
     trace variable addr_db(hideexcluded) w Addr_Browse_Exclude_Change
     Addr_LoadDB
 }
@@ -172,7 +210,7 @@ proc Hook_MsgShowAddr {path headervar } {
         return
     }
     Addr_Save [MsgParseFrom $header(0=1,hdr,from)] $path \
-            $header(0=1,hdr,from) 	$header(0=1,hdr,date)
+            $header(0=1,hdr,from)       $header(0=1,hdr,date)
 }
 
 #
@@ -183,7 +221,7 @@ proc Hook_FolderChangeAddr {newfolder} {
     global addr_db
 
     if {$addr_db(changed) && $addr_db(checkpoint_on_folder_change)} {
-		Addr_SaveFile
+        Addr_SaveFile
     }
 }
 
@@ -193,7 +231,7 @@ proc Hook_FolderChangeAddr {newfolder} {
 # database here.
 #
 proc Addr_CheckPoint {} {
-	Addr_SaveFile
+    Addr_SaveFile
 }
 
 #####
@@ -230,7 +268,7 @@ proc SaveTo { w } {
     # Only allows expansion on addressable header lines.
     if [regexp -nocase {^(to: *|resent-to: *|cc: *|resent-cc: *|bcc: *|dcc: *)(.*)} $line t0 t1 t2] {
         ##  AddrDebug  "  matched! keep is \"$t1\", partial name=\"$t2\""
-		if [regexp -indices ",?.*, *" $t2 t0] {
+        if [regexp -indices ",?.*, *" $t2 t0] {
             set t0 [lindex $t0 end]
             ##  AddrDebug "got comma at $t0"
             set t3 [string range $t2 0 $t0]
@@ -293,9 +331,29 @@ proc Addr_SaveFile { {force 0} } {
     if {0 == $addr_db(changed) && 0 == $force} return
 
     Exmh_Status "Saving address database..."
+    if {$addr_db(remove_days) == ""} {
+        set expiration 0
+     
+    } else {
+        set expiration [expr [clock seconds] - (60*60*24*$addr_db(remove_days))]
+    }
     set fd [open "$homeDir/.exmh_addr_tmp" w]
     foreach i [array names addr_list] {
-        puts $fd [list set addr_list($i) $addr_list($i)]
+       if {[catch {if {$addr_db(remove_entries) == 1 &&
+                       $addr_db(enabled) == 1 &&
+                       $expiration > 0 &&
+                       [lindex $addr_list($i) 1] != "" &&
+                       [clock scan [lindex $addr_list($i) 1]] < $expiration} {
+                      unset addr_list($i) 
+                  } else {
+                     puts $fd [list set addr_list($i) $addr_list($i)]
+                  }}]!= 0} {
+          if {$addr_db(remove_invalid_date)} {
+             unset addr_list($i)
+          } else {
+             puts $fd [list set addr_list($i) $addr_list($i)]
+          }
+       }
     }
     close $fd
     #  the first time the address file won`t exist yet...
@@ -364,9 +422,9 @@ proc Addr_Entry_ToggleExcluded {key} {
 
 # Format an address with full name.  The result may be either
 #
-#	address (full name)
+#       address (full name)
 # or
-#	full name <address>
+#       full name <address>
 #
 # depending on the state of the standard_address_format flag.
 proc Addr_Entry_FormatForMail {key} {
@@ -474,10 +532,10 @@ proc Addr_Save {from path rawfrom date {forcesave "not"}} {
         Exmh_Status "Updating address \"$from\"."
         if {[string length [Addr_ParseFrom $rawfrom]] == 0} {
             set newone [list $path $date  \
-					[lindex $addr_list($from) 2] $rawfrom [Addr_Entry_IsExcluded $from]]
+                    [lindex $addr_list($from) 2] $rawfrom [Addr_Entry_IsExcluded $from]]
         } else {
             set newone [list $path $date  \
-					[Addr_ParseFrom $rawfrom] $rawfrom [Addr_Entry_IsExcluded $from]]
+                    [Addr_ParseFrom $rawfrom] $rawfrom [Addr_Entry_IsExcluded $from]]
         }
     } else {
         Exmh_Status "Saving address \"$from\"."
@@ -501,12 +559,15 @@ proc Addr_KeyExpand { w } {
     global addr_db
 
     AddrDebug "Addr_KeyExpand: w=$w $addr_db(searchlist)"
-    set line [string trim [$w get {insert linestart} {insert lineend}]]
+    set line [string trimright [$w get {insert linestart} {insert lineend}]]
     ##  AddrDebug "  got line \"$line\""
     # Only allows expansion on addressable header lines.
-    if [regexp -nocase {^(to: *|resent-to: *|cc: *|resent-cc: *|bcc: *|dcc: *)(.*)} $line t0 t1 t2] {
+    #old regexp:  regexp -nocase {^(to: *|resent-to: *|cc: *|resent-cc: *|bcc: *|dcc: *)(.*)} $line t0 t1 t2
+    if {[$w compare insert <= hlimit] && \
+            [regexp -nocase {^([-a-z]+: *)?(.*)} $line t0 t1 t2]} {
         ##  AddrDebug  "  matched! keep is \"$t1\", partial name=\"$t2\""
-        if [regexp -indices ",?.*, *" $t2 t0] {
+        if {[regexp -indices ",?.*, *" $t2 t0] || \
+                [regexp -indices "^ +" $t2 t0]} {
             set t0 [lindex $t0 end]
             ##  AddrDebug "got comma at $t0"
             set t3 [string range $t2 0 $t0]
@@ -516,21 +577,21 @@ proc Addr_KeyExpand { w } {
         }
         if {[string compare $addr_db(lastfound) $t2] != 0 \
                 || $addr_db(curmethod) >= [llength $addr_db(searchlist)]} {
-            Exmh_Status "Reseting start method"
+            Exmh_Status "Resetting start method"
             catch {destroy $w.addrs}
             set addr_db(expansion) {}
             set addr_db(curmethod) 0
             set addr_db(laststring) $t2
         } else {
             if {$addr_db(curmethod) != 0} {
-            	set t2 $addr_db(laststring)
+                set t2 $addr_db(laststring)
             }
         }
         foreach proc [lrange $addr_db(searchlist) $addr_db(curmethod) end] {
             incr addr_db(curmethod)
             Exmh_Status "$proc $t2"
-			set result [busy $proc $t2]
-			if {[string compare $result ""] == 0} continue
+            set result [busy $proc $t2]
+            if {[string compare $result ""] == 0} continue
             if {[llength $result] == 1} {
                 # unique match
                 $w delete  {insert linestart} {insert lineend}
@@ -549,23 +610,23 @@ proc Addr_KeyExpand { w } {
                     $w insert insert [format "%s%s" $t1 $new]
                 }
             }
-			break
+            break
         }
     } else {
-        Exmh_Status "Error in name expansion: not on To: field"
+        Exmh_Status "Error in name expansion: not on supported field"
         return
     }
 }
 
 proc Alias_Lookup {n} {
     global aliases
-	Aliases_Load
-	set t2 [string trim $n]
-	if {[string length [array names aliases $t2]] == 0} {
-		Exmh_Status "No match found for \"$t2\""
-		return {}
-	} {
-		Exmh_Status "Found alias: \"$aliases($t2)\""
+    Aliases_Load
+    set t2 [string trim $n]
+    if {[string length [array names aliases $t2]] == 0} {
+        Exmh_Status "No match found for \"$t2\""
+        return {}
+    } else {
+        Exmh_Status "Found alias: \"$aliases($t2)\""
         if {1 == [llength $aliases($t2)] && \
             0 != [string compare "\{" [string range $aliases($t2) 0 0]] } {
             return [list [Addr_Entry_FormatForMail $aliases($t2)]]
@@ -573,8 +634,8 @@ proc Alias_Lookup {n} {
         # Note: cannot use Address_Entry_FormatForMail here since contents
         # of alias is too unpredicatble.  May be a list of names, may be 
         # a preformatted fullname and address.  So send it back as-is
-		return $aliases($t2)
-	}
+        return $aliases($t2)
+    }
 }
 
 proc Addr_FullNameMatch {n}  {
@@ -606,6 +667,67 @@ proc Addr_FullNameMatch {n}  {
     }
 }
 
+    
+proc LDAP_Lookup {n} {
+    global addr_db
+
+    # Make sure the ldap_server variable has been set in the preferences.
+    if { ($addr_db(ldap_server) == {}) || ($addr_db(ldap_searchbase) == {}) } {
+        return {}
+    }
+    
+    Exmh_Status "Querying $addr_db(ldap_server) from $addr_db(ldap_searchbase) with $n..."
+
+    set query "(|(cn=*$n*)(mail=*$n*)(sn=*$n*)(givenname=*$n*))"
+    if [catch {set ldap_results [exec ldapsearch -h [string trim $addr_db(ldap_server)] \
+                                                 -b $addr_db(ldap_searchbase) \
+                                                 "$query" cn mail]} err] {
+        Exmh_Status "Error executing ldapsearch: $err"
+        return {}
+    }
+
+    # The return from ldapsearch will be something like this:
+    #
+    # cn=Lastname, Firstname
+    # mail=foo@nowhere.com
+    #
+    # cn=Anotherlastname, Anotherfirstname
+    # mail=bar@nowhere.com
+    #
+    # ...
+    set result {}
+    foreach i $ldap_results {
+        if [regexp -nocase {^mail=([^,]*)$} $i dummy email] {
+            lappend result "[LDAP_Entry_FormatForMail $email $name]"
+        } elseif [regexp -nocase "^cn=(.*)$" $i dummy tmp] {
+            set name $tmp
+        } else {
+            append name " $i"
+        }
+    }
+    
+    return $result
+}
+
+
+proc LDAP_Entry_FormatForMail { email name } {
+    global addr_db
+    
+    if {$addr_db(standard_address_format)} {
+        set formatted "$email ($name)"
+    } else {
+        # If there are characters in the name that require quoting,
+        # quote the string.
+        if [string match {*[<>.,'*?]*} $name] {
+            set formatted "\"$name\" <$email>"
+        } else {
+            set formatted "$name <$email>"
+        }
+    }
+
+    return $formatted
+}
+
 proc Addr_Lookup { n } {
     global addr_list
 
@@ -652,13 +774,13 @@ proc Addr_ParseFrom { fromline } {
 
     # if it's "xxx <foo@bar>"...
     if [regexp {([^<]*)(<.*>)} $line t1 t2 t3] {
-        #	AddrDebug "  Matched: ( $t1 )( $t2 )( $t3 )"
+        #       AddrDebug "  Matched: ( $t1 )( $t2 )( $t3 )"
         set token [string trim $t2 ]
     } else {
         # nope, try foo@bar (xxx)
-        #	AddrDebug "  Not xxx <foo@bar>, try foo@bar (xxx)"
+        #       AddrDebug "  Not xxx <foo@bar>, try foo@bar (xxx)"
         if [regexp {^([^\(]*)(\(.*\))[^\)]*$} $line t1 t2 t3] {
-            #	    AddrDebug "  Matched: ( $t1 )( $t2 )( $t3 )"
+            #       AddrDebug "  Matched: ( $t1 )( $t2 )( $t3 )"
             set token $t3
         } else {
             # none of the above, give up.
@@ -692,26 +814,39 @@ proc AddrShowDialog {w list} {
 
     catch {destroy $w.addrs}
     set f [frame $w.addrs -bd 4 -relief ridge]
-    set l [listbox $f.lb -bd 4 -width 50 -height 10]
+    frame $f.top -relief flat
+    set l [listbox $f.top.lb -bd 4 -width 50 -height 10]
     bind $l <Any-Double-1> "\
             AddrShowDialogDone $f $l ;\
             break \
             "
-    focus $w.addrs.lb
+    bind $l <Escape> "\
+            AddrShowDialogCancel $f ;\
+            break \
+            "
+    focus $f.top.lb
+    $f configure -cursor left_ptr
     foreach i $list {
         $l insert end $i
     }
-    pack $f.lb -expand true -fill both
+    pack $f.top.lb -expand true -fill both -side left
+    if {[llength $list] > 10} {
+        $f.top.lb configure -yscrollcommand "$f.top.sy set"
+        scrollbar $f.top.sy -width 15 -command [list $l yview]
+        pack $f.top.sy -expand true -fill y -side left
+    }
+    pack $f.top -expand true -fill both
     frame $f.but -bd 10 -relief flat
     pack $f.but -expand true -fill both
-    Widget_AddBut $f.but ok "Done" [list AddrShowDialogDone $f $l] left
+    Widget_AddBut $f.but ok "Done" [list AddrShowDialogDone $f $l] {left filly}
+    Widget_AddBut $f.but can "Cancel" [list AddrShowDialogCancel $f] {right filly}
     Widget_PlaceDialog $w $f
     tkwait window $f
     if [info exists addr_db(expansion)] {
-    	Exmh_Status "returning $addr_db(expansion)"
-    	return $addr_db(expansion)
+        Exmh_Status "returning $addr_db(expansion)"
+        return $addr_db(expansion)
     } else {
-		return {}
+        return {}
     }
 }
 
@@ -732,6 +867,15 @@ proc AddrShowDialogDone {f l} {
     catch {destroy $f}
 }
 
+proc AddrShowDialogCancel {f} {
+    global addr_db
+
+    set addr_db(expansion) ""
+    AddrDebug "selected $addr_db(expansion)"
+    focus [winfo parent $f]
+    catch {destroy $f}
+}
+
 
 proc Addr_Browse { {state normal} } {
     global exwin
@@ -748,7 +892,7 @@ proc Addr_Browse { {state normal} } {
         $f.quit configure -takefocus {} -command {Exwin_Dismiss .addr_br}
 
         # Create the "Selected..." menu (initially disabled)
-        set menu_sel [Widget_AddMenuB $f selmenu "Selected..." {right padx 1} ]
+        set menu_sel [Widget_AddMenuB $f selmenu "Selected..." {right padx 1 filly} ]
         $f.selmenu configure -takefocus {} -state disabled
         set addr_db(selmenu) $f.selmenu
         Widget_AddMenuItem $menu_sel  "Mail To"           \
@@ -763,7 +907,7 @@ proc Addr_Browse { {state normal} } {
                 { Addr_Browse_Selected ViewLastMsg }
 
         # Create the "Database..." menu
-        set menu_db [Widget_AddMenuB $f dbmenu "Database..." {right padx 1} ]
+        set menu_db [Widget_AddMenuB $f dbmenu "Database..." {right padx 1 filly} ]
         $f.dbmenu configure -takefocus {}
         Widget_AddMenuItem $menu_db   "Save"   \
                 { Addr_SaveFile 1 } <Meta-s>
@@ -771,7 +915,10 @@ proc Addr_Browse { {state normal} } {
                 { Addr_Browse_Reload } <Meta-r>
         Widget_AddMenuItem $menu_db   "Sort"   \
                 { Addr_Browse_LoadListbox "Sorting database..." normal } <Meta-t>
-        if { $Addr_debug == 1 }  {	Widget_AddBut $f ldsrc  "LdSrc"  { Addr_Load_Source } }
+        if { $Addr_debug == 1 }  { Widget_AddBut $f ldsrc  "LdSrc"  { Addr_Load_Source } }
+
+        # Create the New button
+        Widget_AddBut $f new   "New"   { Addr_Browse_New }
 
         # Finally, create the Help button
         Widget_AddBut $f help   "Help"   { Help AddrEdit }
@@ -840,12 +987,12 @@ proc Addr_Browse { {state normal} } {
 proc Addr_Browse_TrackSel {} {
     global addr_db
 
-    catch {	;# windows may not exist
-	if { 0 != [string length [$addr_db(win) curselection]] } {
-	    $addr_db(selmenu) configure -state normal
-	} else {
-	    $addr_db(selmenu) configure -state disabled
-	}
+    catch {     ;# windows may not exist
+        if { 0 != [string length [$addr_db(win) curselection]] } {
+            $addr_db(selmenu) configure -state normal
+        } else {
+            $addr_db(selmenu) configure -state disabled
+        }
     }
 }
 
@@ -853,11 +1000,11 @@ proc Addr_Browse_LoadListbox { {ldmsg ""} {state normal}} {
     global addr_db addr_list
 
     if {![info exists addr_db(win)] ||
-	![winfo exists $addr_db(win)]}  return
+        ![winfo exists $addr_db(win)]}  return
 
     if {[catch {regexp -nocase -- $addr_db(filterstring) {}} err]} {
-	Exmh_Status $err	;# bad pattern
-	return
+        Exmh_Status $err        ;# bad pattern
+        return
     }
 
     $addr_db(win) delete 0 end
@@ -874,38 +1021,38 @@ proc Addr_Browse_LoadListbox { {ldmsg ""} {state normal}} {
     }
 
     if {[llength $l]} {
-	if {0 == [string compare "$state" "normal"]} { 
-	    Exmh_Status "$ldmsg sorting names..."
-	}
-	set l [lsort $l]
-	set n 0
-	set whiz [list | \\ - /]
-	set w 0
-	if {[string length $addr_db(filterstring)] > 0} {
-	    foreach i $l {
-		if [regexp -nocase -- $addr_db(filterstring) $i] {
-			$addr_db(win) insert end $i
-		}
-		incr n
-		if { 0==($n%100) } {
-		    if {0 == [string compare "$state" "normal"]} { 
-			Exmh_Status "$ldmsg inserting names... [lindex $whiz $w]"
-		    }
-		    set w [expr {($w+1)%4}]
-		}
-	    }    
-	} else {
-	    foreach i $l {
-		$addr_db(win) insert end $i
-		incr n
-		if { 0==($n%100) } {
-		    if {0 == [string compare "$state" "normal"]} { 
-			Exmh_Status "$ldmsg inserting names... [lindex $whiz $w]"
-		    }
-		    set w [expr {($w+1)%4}]
-		}
-	    }    
-	}
+        if {0 == [string compare "$state" "normal"]} { 
+            Exmh_Status "$ldmsg sorting names..."
+        }
+        set l [lsort $l]
+        set n 0
+        set whiz [list | \\ - /]
+        set w 0
+        if {[string length $addr_db(filterstring)] > 0} {
+            foreach i $l {
+                if [regexp -nocase -- $addr_db(filterstring) $i] {
+                    $addr_db(win) insert end $i
+                }
+                incr n
+                if { 0==($n%100) } {
+                    if {0 == [string compare "$state" "normal"]} { 
+                        Exmh_Status "$ldmsg inserting names... [lindex $whiz $w]"
+                    }
+                    set w [expr {($w+1)%4}]
+                }
+            }
+        } else {
+            foreach i $l {
+                $addr_db(win) insert end $i
+                incr n
+                if { 0==($n%100) } {
+                    if {0 == [string compare "$state" "normal"]} { 
+                        Exmh_Status "$ldmsg inserting names... [lindex $whiz $w]"
+                    }
+                    set w [expr {($w+1)%4}]
+                }
+            }
+        }
     }
     if {0 == [string compare "$state" "normal"]} { 
         Exmh_Status  "$ldmsg done"
@@ -922,7 +1069,7 @@ proc Addr_Browse_Exclude_Change {name element op} {
 }
 
 proc Addr_Browse_Clip {sel} {
-	global addr_db addr_list mhProfile
+        global addr_db addr_list mhProfile
 
     set victim [MsgParseFrom [$addr_db(win) get $sel]]
     set last [lindex $addr_list($victim) 0]
@@ -999,11 +1146,11 @@ proc Addr_Browse_Selected { { op Noop } } {
 
         switch $op {
 
-	    MailTo {
-		append to $sep[Addr_Entry_FormatForMail \
-			[MsgParseFrom [$addr_db(win) get $sel]]]
-		set sep ", "
-	    }
+            MailTo {
+                append to $sep[Addr_Entry_FormatForMail \
+                    [MsgParseFrom [$addr_db(win) get $sel]]]
+                set sep ", "
+            }
 
             Edit {
                 Addr_Browse_Edit $sel
@@ -1038,7 +1185,7 @@ proc Addr_Browse_Selected { { op Noop } } {
         }
     }
     if {[string length $to] > 0} {
-	Msg_CompTo $to
+        Msg_CompTo $to
     }
 
     Addr_Browse_TrackSel
@@ -1081,15 +1228,15 @@ proc Addr_Browse_Edit {sel} {
 
         Widget_AddBut $f save   "Save"            "Addr_Edit_Save $t $sel"
         Widget_AddBut $f delete "Delete"          "Addr_Edit_Delete $t $sel"
-        Widget_AddBut $f last	"ViewLastMsg"	  "Addr_Browse_Clip $sel"
-        set e [Widget_AddBut $f ignore	"Exclude" "Addr_Edit_Exclude $t $sel"]
+        Widget_AddBut $f last   "ViewLastMsg"     "Addr_Browse_Clip $sel"
+        set e [Widget_AddBut $f ignore  "Exclude" "Addr_Edit_Exclude $t $sel"]
 
-        set n [Addr_LabelledTextField $t.name		"Full Name"	12 "Addr_Edit_Save $t $sel" ]
-        set a [Addr_LabelledTextField $t.address	"Address"	12 "Addr_Edit_Save $t $sel" ]
-        set l [Addr_LabelledTextField $t.lastMsg	"Last Message"	12 "Addr_Edit_Save $t $sel" ]
-        set d [Addr_LabelledTextField $t.date		"Date"		12 "Addr_Edit_Save $t $sel" ]
+        set n [Addr_LabelledTextField $t.name     "Full Name"     12 "Addr_Edit_Save $t $sel" ]
+        set a [Addr_LabelledTextField $t.address  "Address"       12 "Addr_Edit_Save $t $sel" ]
+        set l [Addr_LabelledTextField $t.lastMsg  "Last Message"  12 "Addr_Edit_Save $t $sel" ]
+        set d [Addr_LabelledTextField $t.date     "Date"          12 "Addr_Edit_Save $t $sel" ]
         
-        pack $t.name $t.address	 $t.lastMsg  $t.date
+        pack $t.name $t.address  $t.lastMsg  $t.date
 
     }
 
@@ -1097,18 +1244,73 @@ proc Addr_Browse_Edit {sel} {
     $a delete 0 end;    $a insert 0 $addr
     $l delete 0 end;    $l insert 0 $last
     $d delete 0 end;    $d insert 0 $date
-	
+        
     if {$exclude == 1} {
-       	$e config -text "Include"
+        $e config -text "Include"
         AddrDebug "$e config -text Include"
     } else {
-       	$e config -text "Exclude"
+        $e config -text "Exclude"
         AddrDebug "$e config -text Exclude"
     }
 }
 
 
 
+proc Addr_Browse_New {} {
+    global addr_db addr_list
+
+    set t .addr_ed
+    set id 0
+    for {set id 1} {$id < 21} {incr id} {
+        AddrDebug "winfo exists $t$id is [winfo exists $t$id]"
+        if [winfo exists $t$id] continue
+        append t $id
+        break
+    }
+    if [winfo exists $t] {
+        Exmh_Status "Too many editors open, close one or more and try again"
+        return
+    }
+    if [Exwin_Toplevel $t "New DB address" Addr_Ed] {
+
+        set f $t.but
+
+        $t.but.quit configure -text Cancel -command "Addr_Edit_Dismiss $t"
+        $t configure -width 500
+
+        Widget_AddBut $f save "Save"      "Addr_New_Save $t"
+
+        Addr_LabelledTextField $t.name    "Full Name" 12 "Addr_New_Save $t"
+        Addr_LabelledTextField $t.address "Address"   12 "Addr_New_Save $t"
+
+        pack $t.name $t.address
+    }
+}
+
+proc Addr_New_Save {winname} {
+    global addr_db addr_list
+
+    set name [$winname.name.entry get]
+    set addr [$winname.address.entry get]
+    set last ""
+    set date ""
+
+    set index [MsgParseFrom $addr]
+    # NEED TO STUFF new NAME into entry!
+    set addr [format "%s <%s>" $name $index]
+    Exmh_Status "Updating address \"$index\"."
+    set addr_db(changed) 1
+    set addr_list($index) [list $last $date  \
+            [Addr_ParseFrom $addr] $addr] 
+
+    # update browser window...
+    $addr_db(win) insert end [Addr_Entry_FormatForListbox $index]
+
+    # make it all go away so we can redo it next time.
+    Addr_Edit_Dismiss $winname
+}
+  
+  
 proc Addr_Load_Source {}  {
     global env
     # HACK HACK HACK!!!
@@ -1133,7 +1335,7 @@ proc Addr_Edit_Exclude {winname sel} {
     # update browser window...
     $addr_db(win) delete $sel
     if {$addr_db(hideexcluded) == 0 || $exclude == 0} {
-    	$addr_db(win) insert $sel [Addr_Entry_FormatForListbox $victim]
+        $addr_db(win) insert $sel [Addr_Entry_FormatForListbox $victim]
     }
 
     # make it all go away so we can redo it next time.
@@ -1151,7 +1353,7 @@ proc Addr_Edit_Save {winname sel} {
     set index [MsgParseFrom $addr]
     # NEED TO STUFF new NAME into entry!
     set addr [format "%s <%s>" $name $index]
-	Exmh_Status "Updating address \"$index\"."
+    Exmh_Status "Updating address \"$index\"."
     set addr_db(changed) 1
     set addr_list($index) [list $last $date  \
             [Addr_ParseFrom $addr] $addr] 
@@ -1189,5 +1391,3 @@ proc Addr_Edit_Dismiss { winname } {
 # done loading...
 #
 if {$Addr_debug} { puts stdout "done." }
-
-
