@@ -28,15 +28,26 @@ proc Background_Init {} {
 processing is done in a separate process or not.  A
 separate process causes less interference with the user
 interface, but might take up more machine resources."} \
-	{ exmh(background) bgAction {CHOICE off count msgchk flist inc} {Background processing}
+	{ exmh(sendHack) sendHack ON {Keep xhost list clear}
+"The Tk send command will stop working if hosts are added to
+your xhost list, even if you are using Xauthority.  This option
+replaces the send command with a version that clears out the
+xhost list if hosts are found on it."} \
+	{ exmh(background) bgAction {CHOICE off count msgchk flist inc hook} {Background processing}
 "exmh can periodically do some things for you:
 count - count new messages sitting in your spool file.
 msgchk - run the MH msgchk program.
 flist - check for new mail in all folders.
 inc - just like clicking the Inc button yourself.
+hook - suppply your own Hook_Background procedure.
 off - do nothing in the background."} \
 	{ exmh(bgPeriod) bgPeriod 10 {Period (minutes)}
 "How often to do background task"} \
+	{exmh(bgNews) bgNews OFF {Background News Retrieval}
+"Retrieve News via NNTP in the background.  See the Preferences for 
+NNTP support for more information"} \
+	{ exmh(bgNewsPeriod) bgNewsPeriod 60 {Period (minutes)}
+"How often to do background News Retrieval"} \
 	[list exmh(spool) bgSpool $spool {Mail spool pathname} \
 "Pathname for the mail spool file that gets new messages."] \
     ]
@@ -44,9 +55,22 @@ off - do nothing in the background."} \
     # I can hack around it by tracing the bgPeriod, which is always
     # set by Preferences because it is an entry
     trace variable exmh(bgPeriod) w BackgroundFixup
+    if {$exmh(bgPeriod) <= 0} {
+        set exmh(bgPeriod) 10
+    }
     set exmh(lastBackground) $exmh(background)
+
+    set exmh(timeSinceBgNews) -1
+
+    if [catch {User_Layout} err] {
+        puts stderr "User_Layout: $err"
+    }
+
     set background(lastMsgChk) {}
     set exmh(sendErrors) 0
+    if {$exmh(sendHack)} {
+	source $exmh(library)/send.tcl
+    }
 }
 proc Background_Startup {} {
     global exmh inc
@@ -70,17 +94,12 @@ proc Background_Startup {} {
 	Background_DoPeriodic
 	return
     }
-    global tk_version mh_path
-    set prog exmh-bg
-    set cmd [list exec $prog [winfo name .] $exmh(library) $mh_path &]
+    global mh_path argv0 wish
+    set prog ${argv0}-bg
+    Exmh_Status "Starting: $prog"
+    set cmd [list exec $wish -f $prog [winfo name .] $exmh(library) $mh_path &]
     if [catch {
-	if {$tk_version <= 3.2} {
-	    eval $cmd
-	    set pid [exec ps | grep exmh-bg | sed /grep/d]
-	    set pid [lindex $pid 0]
-	} else {
-	    set pid [eval $cmd]
-	}
+	set pid [eval $cmd]
 	set exmh(bgPid) $pid
 	Exmh_Debug Background_Startup $exmh(background) pid $pid
 	after [expr 10*1000*60] BackgroundCheckup
@@ -91,6 +110,7 @@ proc Background_Startup {} {
 }
 proc BackgroundCheckup {} {
     global exmh
+    Exmh_Debug BackgroundCheckup
     if [BgLostPid $exmh(bgPid) exmh-bg] {
 	catch {unset exmh(bgInterp)}
 	Exmh_Debug Restarting exmh-bg
@@ -103,6 +123,9 @@ proc Background_Register { bgInterp {bgPid junk} } {
     # Invoked by the background interpreter so we can talk back to it
     global exmh
     set exmh(bgInterp) $bgInterp
+    if {$bgPid != "junk"} {
+	set exmh(bgPid) $bgPid
+    }
     Exmh_Debug "Background interp is $bgInterp, pid $exmh(bgPid)"
 
     # Bundle up some parameters that could be overridden on the
@@ -114,20 +137,55 @@ proc Background_Register { bgInterp {bgPid junk} } {
     return $results
 }
 proc Background_Cleanup {} {
-    global exmh
-    catch {send $exmh(bgInterp) Exmhbg_Done}
-    catch {exec kill $exmh(bgPid)}
+    global exmh bgaction
+    if [catch {send $exmh(bgInterp) Exmhbg_Done [winfo name .]}] {
+	catch {exec kill $exmh(bgPid)}
+    }
+    catch {
+	foreach action [array names bgaction] {
+	    BackgroundComplete $action
+	}
+    }
+    foreach interp [winfo interps] {
+	if {[string match $exmh(name)-bg* $interp]} {
+	    catch {send $interp Exmhbg_Done [winfo name .]}
+	}
+    }
 }
 proc Background_DoPeriodic {} {
     global exmh
     Exmh_Debug Background_DoPeriodic $exmh(background)
-    case $exmh(background) {
-	"count" { after [expr $exmh(bgPeriod)*1000*60] BackgroundCount }
-	"msgchk"  { after [expr $exmh(bgPeriod)*1000*60] BackgroundMsgChk }
-	"inc"     { after [expr $exmh(bgPeriod)*1000*60] BackgroundInc }
-	"flist"   { after [expr $exmh(bgPeriod)*1000*60] BackgroundFlist }
-	"off"     { after [expr $exmh(bgPeriod)*1000*60] Background_DoPeriodic }
+    switch -- $exmh(background) {
+	"count"  { set bgProc BackgroundCount }
+	"msgchk" { set bgProc BackgroundMsgChk }
+	"inc"    { set bgProc BackgroundInc }
+	"flist"  { set bgProc BackgroundFlist }
+	"hook"	 {
+	    set bgProc [info commands Hook_Background]
+	    if {[string length $bgProc] == 0} {
+		Exmh_Status "Hook_Background undefined (hook background option)"
+		set exmh(background) off
+	    }
+	}
+	default { set bgProc {} }
     }
+    if {[string length $bgProc] != 0} {
+	if [catch $bgProc err] {
+	    Exmh_Debug $bgProc $err
+	}
+    }
+
+    if {$exmh(bgNews)} {
+	if {(($exmh(timeSinceBgNews) == -1) ||
+	($exmh(timeSinceBgNews) > ($exmh(bgNewsPeriod) * 60)))} {
+	    GetNews
+	    set exmh(timeSinceBgNews) 0
+	} 
+	
+	incr exmh(timeSinceBgNews) [expr int($exmh(bgPeriod) * 60)]
+    }
+
+    after [expr int($exmh(bgPeriod)*1000*60)] Background_DoPeriodic
 }
 proc Background_Off {} {
     global exmh
@@ -136,42 +194,38 @@ proc Background_Off {} {
 proc BackgroundFixup { args } {
     global exmh
     Exmh_Debug BackgroundFixup $exmh(lastBackground) $exmh(background)
+    if {[catch {expr $exmh(bgPeriod)*1000*60}] ||
+	 ($exmh(bgPeriod) <= 0)} {
+	set exmh(bgPeriod) 10
+    }
     if {$exmh(background) != $exmh(lastBackground)} {
 	Background_Startup
 	set exmh(lastBackground) $exmh(background)
     }
 }
+
 proc BackgroundMsgChk {} {
     global exmh env background
-    if {$exmh(background) != "msgchk"} {
-	return
-    }
     set result [Mh_MsgChk]
     if {$result != $background(lastMsgChk)} {
+	BgRPC BackgroundMsgChkInner $result
 	Exmh_Status $result
 	set background(lastMsgChk) $result
     }
-    if [catch {after [expr $exmh(bgPeriod)*1000*60] {BackgroundMsgChk}} msg] {
-	puts stderr "BackgroundMsgChk catch: $msg"
-    }
 }
-proc Background_Counter {} {
-    global exmh
-    if {$exmh(background) != "count"} {
-	set exmh(background) "count"
-	BackgroundCount
+proc BackgroundMsgChkInner {result} {
+    global background exmh
+    Exmh_Status $result
+    set background(lastMsgChk) $result
+    switch -glob -- $result {
+	"You have*" {set exmh(numUnInced) "Some"; Flag_Spooled}
+	"You don't*" {set exmh(numUnInced) 0; Flag_NoSpooled}
     }
 }
 proc BackgroundCount {} {
     global exmh env
-    if {$exmh(background) != "count"} {
-	return
-    }
     if ![catch {Mh_MsgCount $exmh(spool)} newmsgs] {
-	BackgroundNewMsgs [string trim $newmsgs]
-    }
-    if [catch {after [expr $exmh(bgPeriod)*1000*60] {BackgroundCount}} msg] {
-	puts stderr "BackgroundCount catch: $msg"
+	BgRPC BackgroundNewMsgs [string trim $newmsgs]
     }
 }
 
@@ -186,73 +240,77 @@ proc BackgroundNewMsgs { N } {
 	} else {
 	    set msg "msgs"
 	}
+	set exmh(numUnInced) $N
 	Exmh_Status "You have $N spooled $msg" blue
+	Flag_Spooled
     } else {
+	Flag_NoSpooled
 	if {$exmh(numUnInced) > 0} {
 	    Exmh_Status ""
 	}
-    }
-    set exmh(numUnInced) $N
-}
-proc Background_Inc {} {
-    global exmh
-    if {$exmh(background) != "inc"} {
-	set exmh(background) "inc"
-	BackgroundInc
+	set exmh(numUnInced) $N
     }
 }
 proc BackgroundInc {} {
-    global exmh
-    if {$exmh(background) != "inc"} {
-	return
-    }
     Inc
-    catch {after [expr $exmh(bgPeriod)*1000*60] BackgroundInc}
-}
-proc Background_Flist {} {
-    global exmh
-    if {$exmh(background) != "flist"} {
-	set exmh(background) "flist"
-	BackgroundFlist
-    }
-}
-proc BackgroundFlist {} {
-    global exmh
-    if {$exmh(background) != "flist"} {
-	return
-    }
-    Flist_FindUnseen		;# Update folder highlights
-    BgRPC Inc_PresortFinish	;# Update scan listing
-    catch {after [expr $exmh(bgPeriod)*1000*60] BackgroundFlist}
 }
 
-# Invoke something in the background interpreter
-proc BgAction { args } {
+proc BackgroundFlist {} {
+    Flist_FindUnseen		;# Update folder highlights
+    BgRPC Inc_PresortFinish	;# Update scan listing
+}
+
+# Invoke something in the background interpreter, if it exists
+proc BgAction { tag args } {
     global exmh
+    Exmh_Debug BgAction $tag $args
+    Audit "$tag $args"
     if [info exists exmh(bgInterp)] {
-	if {! [catch {send $exmh(bgInterp) [list after 1 $args]} err]} {
+	BackgroundPending $tag	;# Register outstanding request
+	if ![catch {
+	    send $exmh(bgInterp) [list after 1 [list BgProcess $tag $args]]
+	} err] {
 	    return
 	}
+	BackgroundComplete $tag
 	Exmh_Debug BgAction $err
     }
     eval $args
+    foreach cmd [info commands Hook_Bg$tag*] {
+	$cmd
+    }
+}
+# Run something in the background and report back to the front end
+proc BgProcess { tag cmd } {
+    global exmh
+    if [catch $cmd err] {
+	Exmh_Status $err
+    }
+    if [catch {send $exmh(interp) [list BackgroundComplete $tag]} err ] {
+	catch {puts stderr "exmh-bg: BackgroundComplete($tag) failed: $err"}
+	Exmh_Status $err
+    }
 }
 
 # Invoke a routine in the UI interpreter, if it exists, else ourselves.
-# The idea is that the command might or might not be invoked
-# from the background interpreter
+# If there is no separate background process, then
+# exmh(interp) does not exist, and we just eval the command
+# in the current process, which is already the UI.
 proc BgRPC { args } {
     global exmh
+    if [info exists exmh(dead)] {
+	return
+    }
     set check [info exists exmh(pid)]
     if [info exists exmh(interp)] {
 	# Send command to main, front-end interpreter
 	set fail {}
-	if {$check && [BgLostPid $exmh(pid) exmh]} {
-	    # Front-end died or may have restarted - bail out
-	    set fail "process $exmh(pid)"
-	} else {
+#	if {$check && [BgLostPid $exmh(pid) exmh]} {
+#	    # Front-end died or may have restarted - bail out
+#	    set fail "process $exmh(pid)"
+#	} else {
 	    if [catch {send $exmh(interp) $args} err] {
-		case $err {
+		switch -- $err {
 		    {remote\ interpreter\ did\ not\ respond} {
 			if {$check && [BgLostPid $exmh(pid) exmh]} {
 			    set fail "process $exmh(pid)"
@@ -261,9 +319,20 @@ proc BgRPC { args } {
 		    {no\ registered\ interpeter*} {
 			set fail "interp $exmh(interp)"
 		    }
+		    {no\ application\ named*} {
+			set fail "interp $exmh(interp)"
+		    }
+		    {} {
+			set fail "interp $exmh(interp)"
+		    }
+		    default {
+			#puts stderr "BgRPC: $args: $err"
+		    }
 		}
+	    } else {
+		return $err
 	    }
-	}
+#	}
 	if {"$fail" != ""} {
 	    unset exmh(interp)
 	    catch {puts stderr "exmh-bg: lost UI $fail"}
@@ -271,16 +340,13 @@ proc BgRPC { args } {
 	}
     } else {
 	# Eval in main, front-end interpreter
-	eval $args
+	uplevel #0 $args
     }
 }
 proc BgLostPid { pid {name notused} } {
-    global exmh
-    if [catch {eval exec $exmh(psCmd) $pid} err] {
-	if [string match {[Uu]*} $err] {
-	    catch {puts stderr "BgLostPid $err"}
-	    Exmh_Debug BgLostPid $exmh(psCmd) $err
-	}
+    global exmh ps
+    if [catch {PsByID $pid} err] {
+	catch {puts stderr "BgLostPid $ps(cmd) $ps(pflag) $pid: $err"}
 	return 1
     } else {
 	foreach line [split $err \n] {
@@ -288,6 +354,7 @@ proc BgLostPid { pid {name notused} } {
 		return 0
 	    }
 	}
+	catch {puts stderr "BgLostPid pid $pid: cannot find in ps output"}
 	return 1
     }
 }
@@ -308,35 +375,37 @@ proc BgLostPidOld { pid {name notused} } {
     }
 }
 # Improved version of BgLostPid thanks to Scott Hammond
-set ps ps
-set ps_opt ""
-proc BgLostPidClever { pid {name notused} } {
-    global ps ps_opt
-    if [catch "exec $ps $ps_opt $pid" err] {
-        #puts stderr "ps error: $err"
-        if [string match {[Uu]sage:*} $err] {
-            # got usage, so ps must be right, -p should also be right
-            set ps_opt "-p"
-            return [catch {exec $ps -p $pid}]
-        } elseif [string match {*can't find controlling terminal} $err] {
-            if {"$ps" == "ps"} {
-                set ps "/bin/ps"
-            } elseif {"$ps" == "/bin/ps"} {
-                set ps "/usr/ucb/ps"
-            } else {
-                return 1
-            }
-            return [BgLostPid $pid $name]
-        } else {
-            return 1
-        }
-    } else {
-        foreach line [split $err \n] {
-            if {[string compare [lindex $line 0] $pid] == 0} {
-                return 0
-            }
-        }
-        return 1
+if {0} {
+    set ps ps
+    set ps_opt ""
+    proc BgLostPidClever { pid {name notused} } {
+	global ps ps_opt
+	if [catch "exec $ps $ps_opt $pid" err] {
+	    #puts stderr "ps error: $err"
+	    if [string match {[Uu]sage:*} $err] {
+		# got usage, so ps must be right, -p should also be right
+		set ps_opt "-p"
+		return [catch {exec $ps -p $pid}]
+	    } elseif [string match {*can't find controlling terminal} $err] {
+		if {"$ps" == "ps"} {
+		    set ps "/bin/ps"
+		} elseif {"$ps" == "/bin/ps"} {
+		    set ps "/usr/ucb/ps"
+		} else {
+		    return 1
+		}
+		return [BgLostPid $pid $name]
+	    } else {
+		return 1
+	    }
+	} else {
+	    foreach line [split $err \n] {
+		if {[string compare [lindex $line 0] $pid] == 0} {
+		    return 0
+		}
+	    }
+	    return 1
+	}
     }
 }
 
@@ -344,71 +413,7 @@ proc Background_Preferences {} {
     # Tell the background interpreter to update its per-user settings
     global exmh
     if [info exists exmh(bgInterp)] {
-	catch {send $exmh(bgInterp) [list PreferencesReset]}
-    }
-}
-
-proc Background_Rmm { rmmProc folder msgs } {
-    global exmh
-    global background
-    if [info exists exmh(bgInterp)] {
-	if [catch {
-	    send $exmh(bgInterp) [list after 1 \
-				[list BackgroundRmm $rmmProc $folder $msgs]]
-	    BackgroundPending "remove"
-	} err] {
-	    Exmh_Debug Background_Rmm $err
-	    catch {$rmmProc $folder $msgs}
-	}
-    } else {
-	$rmmProc $folder $msgs
-    }
-}
-proc BackgroundRmm { rmmProc folder msgs } {
-    global exmh
-    if [catch {$rmmProc $folder $msgs} err] {
-	Exmh_Status "$rmmProc: $err"
-    }
-    if [catch {
-	send $exmh(interp) [list BackgroundComplete "remove"]
-    } err] {
-	# The UI is probably scanning a new folder
-	Exmh_Status $err
-    }
-}
-proc Background_Refile { refileProc srcfolder msgs folder ok scan new} {
-    global exmh
-    if [info exists exmh(bgInterp)] {
-	if [catch {
-	    send $exmh(bgInterp) [list after 1 \
-		[list BackgroundRefile $refileProc $srcfolder $msgs $folder $ok $scan $new]]
-
-	    BackgroundPending "refile $folder"
-	} err] {
-	    Exmh_Debug Background_Refile $err
-	    catch {$refileProc $srcfolder $msgs $folder}
-	}
-    } else {
-	$refileProc $srcfolder $msgs $folder
-	if {$ok} {
-	    Scan_Move $folder $scan $new
-	}
-    }
-}
-proc BackgroundRefile { refileProc srcfolder msgs folder ok scan new } {
-    global exmh
-    if [catch {$refileProc $srcfolder $msgs $folder} err] {
-	Exmh_Status "$refileProc: $err"
-    } else {
-#	Exmh_Status "Background $refileProc folder done"
-    }
-    if [catch {
-	send $exmh(interp) [list BackgroundComplete "refile $folder"]
-    } err ] {
-	Exmh_Status $err
-    }
-    if {$ok} {
-	Scan_Move $folder $scan $new
+	catch {send $exmh(bgInterp) [list Preferences_Reset]}
     }
 }
 
@@ -421,21 +426,30 @@ proc BackgroundComplete { action } {
     global bgaction
     catch {unset bgaction($action)}
     Exmh_Debug BackgroundComplete $action
+    if [regexp {Refile (.*)} $action x folder] {
+	global exmh
+	if {[string compare $exmh(folder) $folder] == 0} {
+	    Exmh_Status "Updating scan listing"
+	    Scan_FolderUpdate $folder
+	}
+    }
     if {[Background_Outstanding] == {}} {
 	Exmh_Status "background actions complete"
     }
+    foreach cmd [info commands Hook_Bg$action*] {
+	$cmd
+    }
 }
 proc Background_Outstanding {} {
     global bgaction background
     if [catch {array names bgaction} actions] {
-	set background(complete) 1
-	return {}
-    } else {
-	if {$actions == {}} {
-	    set background(complete) 1
-	}
-	return $actions
+	set actions {}
     }
+    if {$actions == {}} {
+	set background(complete) 1
+	catch {destroy .ftoc.t.abort}
+    }
+    return $actions
 }
 proc Background_Wait {} {
     global background
@@ -443,6 +457,20 @@ proc Background_Wait {} {
     set pending [Background_Outstanding]
     if {$pending != {}} {
 	Exmh_Status "waiting... $pending"
+	catch {
+	button .ftoc.t.abort -text "Don't Wait" -command Background_Reset
+	place .ftoc.t.abort -relx .5 -rely .5 -anchor c
+	}
 	tkwait variable background(complete)
+	catch {destroy .ftoc.t.abort}
     }
 }
+
+proc Background_Reset {} {
+    global bgaction
+    foreach act [array names bgaction] {
+	Exmh_Status "Clearing $act"
+    }
+    unset bgaction
+    Background_Outstanding
+}
