@@ -315,9 +315,10 @@ unquotable MIME type." {
 "If you do not enabled quoting, exmh will not create any quote file
 when replying." }
     {quote(filename) quoteFilename @ {Quote file name}
-"The filename of the quote file relative to the exmh home directory,
-which is usually the user's home directory.  The default is @, I
-have mine set to Mail/@." }
+"The filename of the quote file, either with an absolute path or
+relative to the exmh startup directory.  The default is @, but you may
+want to pick another location such as Mail/@ if the exmh startup
+directory is world readable." }
     {quote(symlink) quoteSymlink OFF {Only symlink}
 "In previous versions of exmh, the quote file was only a symlink to
 the message being replied to.  Select this if you want to override
@@ -523,38 +524,70 @@ This set of preferences controls the general behavior of all the
 PGP modules." {
     {pgp(seditpgp) pgpSeditPgp OFF {Sedit PGP passphrase}
 "Turning this on provides you with a PGP passphrase field in the sedit
-window so that you will not be prompted with the passphrase prompt.  
-Changing this value will require that you exit and re-enter exmh if 
-you've already composed email." }
+window so that you will not be prompted with the passphrase prompt." }
     {pgp(seditpgpextras) pgpSeditPgpExtras OFF {Sedit PGP extras info}
 "Turning this on gives extra information about what key and PGP
 version are currently selected.  Users with multiple PGP keys or PGP
 versions may find it helpful.  Changing this value will require that 
 you exit and re-enter exmh if you've already composed email." }
-    {pgp(sign) pgpSign ON {Sign outgoing messages}
-"If this is turned on, outgoing messages will be signed." }
-    {pgp(encrypt) pgpEncrypt OFF {Encrypt outgoing messages}
-"If this is turned on, outgoing messages will be encrypted." }
-    {pgp(mime) pgpMime ON {Use Mime for PGP}
-"If this is turned on, outgoing PGP messages will use Mime for encapsulation." }
-    {pgp(clearsign) pgpClearSign ON {Sign messages in the clear}
-"If this is turned on, messages will be signed so that they can be read by
-non-PGP mail readers." }
+    {pgp(sign) pgpSign {CHOICE none standard detached clearsign encryptsign} {Signature options}
+"There are multiple ways to sign a message.
+    None: Do not sign outgoing messages.
+    Standard: Sign outgoing messages.
+    Detached: Use MIME encapsulation for signatures.
+    Clearsign: Sign messages so that they can be read by non-PGP mail readers.
+    Encryptsign: Sign and encrypt in a single operation.
+This can be changed on the fly from the sedit window." }
+    {pgp(encrypt) pgpEncrypt OFF {Encrypt-only outgoing messages}
+"If this is turned on, outgoing messages will be encrypted but not signed." }
     {pgp(format) pgpFormat {CHOICE pm plain app} {Format to encode PGP}
 "There are multiple standards for PGP encoding.
     Pm:     Use the multipart/pgp standard (This is the preferred standard)
     Plain:  No MIME headers at all
     App:    Use the now deprecated application/pgp standard.
-This can be changed on the fly from the sedit window" }
-    {pgp(version) pgpVersion {CHOICE pgp pgp5 gpg} {Version of PGP for new messages}
+This can be changed on the fly from the sedit window." }
+    }
+
+    Preferences_Add "General PGP Interface" "" [list \
+    [list pgp(version) pgpVersion [concat CHOICE $setup] \
+    {Version of PGP for new messages} \
 "There are multiple versions of the PGP program.
     PGP:    Pretty Good Privacy, Version 2
     PGP5:   Pretty Good Privacy, Version 5
     GPG:    GNU Privacy Guard
-This can be changed on the fly from the sedit window" }
+This can be changed on the fly from the sedit window." ] ]
+
+    Preferences_Add "General PGP Interface" "" {
+    {pgp(keeppass) pgpKeepPass ON {Keep PGP passphrase}
+"Exmh tries to remember your passphrase between PGP
+invocations. But the passphrase is then kept in a global
+variable, which is not safe, because of \"send\"'s power.
+If you turn this feature off, exmh will use xterm to run
+pgp so that it doesn't have to deal with the passphrase at all." }
+    {pgp(echopass) pgpEchoPass  ON {Echo '*' when typing pass}
+"If you have pgpKeepPass on, Exmh will prompt for your passphrase.
+A * will be echoed for every character typed depending on this option." }
+    {pgp(grabfocus) pgpGrabFocus  ON {Passphrase dialog grabs input focus}
+"When exmh prompts for the PGP passphrase it will globally grab input 
+focus if this is on.  Some users like it because they don't need to
+select the popup dialog or because it lessens the risk they will type
+their passphrase in the wrong window.  It annoys or does not work for
+other people." }
     {pgp(passtimeout) pgpPassTimeout 60 {Minutes to cache PGP passphrase}
 "Exmh will clear its memory of PGP passphrases after
-this time period, in minutes, has elapesed." }
+this time period, in minutes, has elapsed." }
+    }
+
+    # Make sure we don't inherit a bad pgp(version) from a previous setup
+    if ![set pgp($pgp(version),enabled)] {
+	set pgp(version) [lindex $setup 0]
+	Preferences_Tweak pgp(version)
+    }
+
+    # Fix up change in pgp(sign) value
+    switch $pgp(sign) {
+	1 {set pgp(sign) standard}
+	0 {set pgp(sign) none}
     }
 
     # And now load the version specific stuff
