@@ -18,8 +18,11 @@ proc Exwin_Init {} {
 Contrained scrolling keeps the last line of text in a window stuck
 to the bottom of the window.  This can be set independently for
 the scan listing (FTOC)." {
-	{exwin(scrollbarSide) scrollbarSide {CHOICE left right} {Scrollbar side}
-"Which side the scrollbars appear on.  This
+	{exwin(scrollbarSide) scrollbarSide {CHOICE left right} {Vertical Scrollbar Side}
+"Which side the vertical scrollbars appear on.  This
+only takes effect after you restart exmh."}
+	{exwin(hscrollbarSide) hscrollbarSide {CHOICE none top bottom} {Horizontal Scrollbar Side}
+"Which side the horizontal scrollbars appear on.  This
 only takes effect after you restart exmh."}
 	{exwin(scrollSpeed) scrollSpeed 1 {Drag-Scroll speed}
 "How fast things scroll when you drag a text widget
@@ -27,6 +30,10 @@ around with the (by default) middle button."}
 	{exwin(scrollAccel) scrollAccel 4 {Drag-Scroll acceleration}
 "How fast things scroll when you drag a text widget
 around with the (by default) middle button with Shift depressed."}
+  	{exwin(wheelEnabled) wheelEnabled OFF "Wheel Mouse"
+"Enables the use of wheel mice, assuming proper
+X Server support (ie, button 4 + 5)
+You must restart exmh for this option to take effect."}
     {widgetText(constrained) textConstrainScroll OFF "Constrained Scrolling"
 "Constrained scrolling clamps the last line
 of text to the bottom of the text widget."} 
@@ -64,10 +71,26 @@ windows."}
     set exwin(ftextLinesSave) $exwin(ftextLines)
     trace variable exwin(ftextLines) w ExwinFixupFtextLines
 #    trace variable exwin(mtextLines) w ExwinFixupMtextLines
+
+    if {$exwin(wheelEnabled)} {
+	mscroll TScroll 5
+    }
+
     if ![info exists exwin(toplevels)] {
 	set exwin(toplevels) [option get . exwinPaths {}]
     }
 }
+
+proc mscroll {bindtag num} {
+    bind $bindtag <Button-5> [list %W yview scroll $num units]
+    bind $bindtag <Button-4> [list %W yview scroll -$num units]
+    bind $bindtag <Shift-Button-5> [list %W yview scroll 1 units]
+    bind $bindtag <Shift-Button-4> [list %W yview scroll -1 units]
+    bind $bindtag <Control-Button-5> [list %W yview scroll 1 pages]
+    bind $bindtag <Control-Button-4> [list %W yview scroll -1 pages]
+}
+
+
 proc ExwinFixupFtextLines { args } {
     global exwin
     Exmh_Debug ExwinFixupFtextLines $exwin(ftextLines)
@@ -113,6 +136,11 @@ proc Exwin_Layout {} {
     set exwin(ftext) [Widget_Text [Widget_Frame . ftoc Ftoc $fixed] \
 				$exwin(ftextLines)]
     Ftoc_Bindings $exwin(ftext)
+
+    if {$exwin(wheelEnabled)} {
+	mscroll $exwin(ftext) 1
+    }
+
     Ftoc_ColorConfigure $exwin(ftext)
 
     # Frame for faces, status, message buttons
@@ -146,19 +174,7 @@ proc Exwin_Layout {} {
     Msg_Setup $exwin(mtext)
     Bindings_Main $exwin(mtext)
 
-    #
-    # Focus handling.  The message display is the default focus
-    # because all the command keys are bound there.
-    # The <FocusIn> binding is for when the user has "focus-follows-mouse"
-    # and moves the mouse into the main window from, say, the sedit window.
-
-    global tk_version
-    if {$tk_version < 4.0} {
-	focus default $exwin(mtext)
-	bind . <FocusIn> [list focus $exwin(mtext)]
-    } else {
-	focus $exwin(mtext)
-    }
+    focus $exwin(mtext)
 }
 proc ExwinFtocMsgBoundary {frame} {
     global exwin
@@ -330,30 +346,19 @@ proc ExwinLineHeight {w} {
 }
 proc Exwin_FullFtoc {} {
     global exwin
-    global tk_version
     if ![info exists exwin(fullFtoc)] {
 	set exwin(fullFtoc) notFullScreen
     }
     if {$exwin(fullFtoc) == "notFullScreen"} {
 	set exwin(fullFtoc) fullScreen
-	if {$tk_version >= 3.3} {
-	    set exwin(ftocPack) [pack newinfo .msg]
-	    pack forget .msg
-	    $exwin(ftext) configure -height \
+	set exwin(ftocPack) [pack newinfo .msg]
+	pack forget .msg
+	$exwin(ftext) configure -height \
 		[expr $exwin(ftextLines)+$exwin(mtextLines)]
-	} else {
-	    set exwin(ftocPack) {top fill expand}
-	    pack unpack .msg
-	}
     } else {
 	set exwin(fullFtoc) notFullScreen
 	$exwin(ftext) configure -height $exwin(ftextLines)
-	if {$tk_version >= 3.3} {
-	    eval pack .msg $exwin(ftocPack)
-	} else {
-	    pack append . .msg $exwin(ftocPack)
-	}
-
+	eval pack .msg $exwin(ftocPack)
     }
 }
 
@@ -456,44 +461,6 @@ proc Exwin_Dismiss { path {geo ok} } {
     wm withdraw $path
     update idletasks	;# Helps window dismiss
 }
-proc Exwin_ToplevelFocus { toplevel {default none} } {
-    global exwin
-    if {[string compare $default none] != 0} {
-	focus $default
-    }
-    global tk_version
-    if {$tk_version < 4.0} {
-	set exwin(focus,$toplevel) $default
-	bind $toplevel <FocusIn> {ExwinSetFocus %W}
-	bind $toplevel <FocusOut> {ExwinSaveFocus %W}
-    }
-}
-proc ExwinSetFocus { w } {
-    global exwin
-    catch {
-	focus $exwin(focus,$w) ;# could be "none", or empty
-    }
-}
-proc ExwinSaveFocus { w } {
-    global exwin
-    set new [focus]
-    if {[string compare $new none] == 0} {
-	return
-    }
-    if {[string compare [winfo toplevel $new] $w] == 0} {
-	set exwin(focus,$w) $new
-    }
-}
-proc Exwin_Grab { args } {
-    global exwin
-    Exmh_Debug Exwin_Grab $args focus=[focus]
-    if {[string compare [lindex $args 0] release] == 0} {
-	set exwin(grab) {}
-    } else {
-	set exwin(grab) $args
-    }
-    eval grab $args
-}
 proc Exwin_CheckPoint { } {
     global exwin
     if {! $exwin(placeToplevel)} {
