@@ -29,6 +29,7 @@ proc Face_SetPath {} {
 	    # backwards compatibility with old "exmh" script
 	    set faces(set,user) $faces(set)
 	    set faces(set,unknown) $faces(set)
+	    set faces(set,news) $faces(set)
 	}
 	set faces(sets) {user unknown}
     }
@@ -36,6 +37,7 @@ proc Face_SetPath {} {
     # tail component for each set
     set faces(name,user) {$user}
     set faces(name,unknown) unknown
+    set faces(name,news) unknown
 
     set faces(defaultDomain) [string tolower \
 	[string trim $faces(defaultDomain) ". "]]
@@ -43,6 +45,7 @@ proc Face_SetPath {} {
     foreach set $faces(sets) {
 	set faces(path,$set) {}
     }
+    set faces(path,news) {}
     if [info exists env(FACEPATH)] {
 	set faces(base) ""
 	foreach dir [split $env(FACEPATH) :] {
@@ -53,6 +56,7 @@ proc Face_SetPath {} {
 		} else {
 		    FaceAddPath user $dir
 		    FaceAddPath unknown $dir
+		    FaceAddPath news $dir
 		}
 	    }
 	}
@@ -64,6 +68,12 @@ proc Face_SetPath {} {
 		FaceAddPath $set $dir
 	    }
 	}
+	if [info exists faces(set,news)] {
+	    foreach dir $faces(set,news) {
+		if ![file isdirectory $faces(base)$dir] continue
+		FaceAddPath news $dir
+	    }
+	}
     }
 }
 proc FaceAddPath {set dir} {
@@ -79,31 +89,61 @@ proc FaceAddPath {set dir} {
 }
 
 
-proc Face_Show { fromwho {xface {}} } {
-    global faces faceCache
+proc Face_Show { fromwho {xface {}} {ximageurl {}} {newsgrps {}} } {
+    global faces faceCache failedURLs exmh
+
+    set xfaceAvail 0
+    set ximageurlAvail 0
+
+    # Don't do any of this if we're on a slow display
+    if {$exmh(slowDisp) && !$exmh(slowDispFaces)} {
+      return 0
+    }
 
     Face_Delete
 
     # Honor X-Face even if faces is disabled
-    if {[string compare "" $xface] && \
+    if {$faces(xFaceEnabled) && \
+	[string compare "" $xface] && \
 	[string compare "" $faces(xfaceProg)]} {
 
 	if {$faces(rowEnabled) && $faces(defer)} {
 	    DeferWork faces(work) [list FaceXFace $xface [FaceAlloc]]
 	} elseif {[FaceXFace $xface] && !$faces(rowEnabled)} {
-	    return 1
+	    set xfaceAvail 1
 	}
     }
-    if !$faces(enabled) {
+
+    # Honor X-Image-URL even if X-Face was displayed or the faces are
+    # disabled
+    if {$faces(xImageUrl) && [string compare "" $ximageurl]} {
+	if {![info exists failedURLs]
+	    || ([info exists failedURLs]
+		&& [lsearch $failedURLs $ximageurl] == -1)} {
+ 	    if {$faces(rowEnabled) && $faces(defer)} {
+ 		DeferWork faces(work) \
+		    [list UrlDisplayFace $ximageurl [FaceAlloc]]
+ 	    } elseif {[UrlDisplayFace $ximageurl [FaceAlloc]]
+		      && !$faces(rowEnabled)} {
+		set ximageurlAvail 1
+	    }
+	}
+    }
+
+    if {$xfaceAvail || $ximageurlAvail} {
+	return 1
+    }
+
+    if {$faces(enabled!) || !$faces(enabled)} {
 	return 0
     }
 
     # Check for cached lookup result
-    if [info exists faceCache($fromwho)] {
-	if [Face_ShowFace $faceCache($fromwho)] {
+    if [info exists faceCache($fromwho,$newsgrps)] {
+	if [Face_ShowFace $faceCache($fromwho,$newsgrps)] {
 		return 1
 	}
-	unset faceCache($fromwho)
+	unset faceCache($fromwho,$newsgrps)
 	Face_Delete
     }
 
@@ -117,13 +157,23 @@ proc Face_Show { fromwho {xface {}} } {
 	set machine [string tolower $faces(defaultDomain)]
     } elseif {[string first . $machine] == -1} {
       append machine . $faces(defaultDomain)
-   }
+    }
 
     set from [split $machine .]
     set pathlist [FacePathlist $from]
 
 #Exmh_Debug \n$user ==> $pathlist
 
+    set pathlistngfull {}
+    if {[string compare "" $newsgrps]} {
+	set newsgrplist [string tolower [split $newsgrps ,]]
+	foreach ng $newsgrplist {
+	    set ngparts [split $ng .]
+	    set pathlistng [FacePathNGlist $ngparts]
+	    set pathlistngfull [concat $pathlistng $pathlistngfull]
+	}
+    }
+
     # Loop through Face path
 #Tputs lookup: [time {
     set matches {}
@@ -152,7 +202,7 @@ proc Face_Show { fromwho {xface {}} } {
 		    break
 		}
 		set path $dir/$part/$name
-#	Exmh_Debug $path
+#		Exmh_Debug $path
 		# skip non-existent directories
 		if ![file exists $faces(base)$path] continue
 
@@ -166,20 +216,42 @@ proc Face_Show { fromwho {xface {}} } {
 	}
     }
 #   }]
+    eval set tail $faces(name,news)
+    foreach dir $faces(path,news) {
+	set name $tail
+	set map {}
+	foreach part [concat $map $pathlistngfull] {
+#	    if {([string match unknown* $dir] || [string match misc* $dir])
+#		 && [llength $matches]} {
+#		break
+#	    }
+	    set path $dir/$part/$name
+#	    Exmh_Debug $path
+	    # skip non-existent directories
+	    if ![file exists $faces(base)$path] continue
+
+	    foreach suf $faces(suffix) {
+		if [file exists $faces(base)$path/face.$suf] {
+		    lappend matches $path/face.$suf
+		    break
+	        }
+	    }
+	}
+    }
 
 #    Exmh_Debug Faces matches $matches
 
     if !$faces(rowEnabled) {
 	foreach face $matches {
 	    if [Face_ShowFile $face] {
-		set faceCache($fromwho) $face
+		set faceCache($fromwho,$newsgrps) $face
 		Exmh_Status $msg
 		return 1
 	    }
 	}
     # braces around cmdsubst NECESSARY!
     } elseif {[Face_ShowFace $matches]} {
-	set faceCache($fromwho) $matches
+	set faceCache($fromwho,$newsgrps) $matches
 	Exmh_Status $msg
 	return 1
     }
@@ -205,6 +277,19 @@ proc FacePathlist { from } {
     return $pathlist
 }
 
+proc FacePathNGlist { ng } {
+    set path {}
+    set prefix {}
+    set pathlist {}
+    for {set i 0} {$i <= [expr [llength $ng]-1]} {incr i 1} {
+	append path $prefix [lindex $ng $i]
+	set prefix /
+	set pathlist [concat $path $pathlist]
+    }
+    lappend pathlist {}
+    return $pathlist
+}
+
 proc Face_Delete {} {
     global faces
 
@@ -305,7 +390,7 @@ proc FaceShowFile {facefile pane} {
 	set facefile $faces(base)$facefile
     }
     switch -- [file extension $facefile] {
-	.ppm - .pgm - .pbm - .gif {
+	.ppm - .pgm - .pbm - .gif - .xpm {
 	    if [catch {
 # Tputs image create: [time {
 		set image [image create photo -file $facefile -palette $faces(palette)]
@@ -324,15 +409,6 @@ proc FaceShowFile {facefile pane} {
 		return 0
 	    }
 	}
-	.xpm {
-	    if [catch {
-		set image [image create pixmap -file $facefile]
-		$pane config -image $image
-	    } id] {
-		Exmh_Debug $id
-		return 0
-	    }
-	}
 	.xbm {
 	    if [catch {
 		$pane config -bitmap @$facefile
@@ -370,7 +446,7 @@ proc FaceXFace { xface {pane {}}} {
 # Tputs show x-face: [time {
     set ret [FaceShowFile [Env_Tmp]/FACE.[pid].xbm $pane]
 # }]
-    exec rm [Env_Tmp]/FACE.[pid].xbm
+    File_Delete [Env_Tmp]/FACE.[pid].xbm
     Exmh_Status ok
     return $ret
 }
@@ -461,13 +537,10 @@ proc Face_FlushCache {} {
     catch {unset faceCache}
 }
 
-
-
 #
 # Defer work to an after handler [this code should be elsewhere]
 #
 
-if {$tk_version >= 4.0} {
 proc DeferWork {name work {cancel {}}} {
     upvar #0 $name queue
 
@@ -497,48 +570,3 @@ proc DeferWorkProc name {
 	after 20 DeferWorkProc $name
     }
 }
-
-
-} else {
-
-
-proc DeferWork {name work {cancel {}}} {
-    upvar #0 $name queue
-    global serial
-
-    lappend queue [list $work $cancel]
-    if {[llength $queue] == 1} {
-	if [catch {incr serial($name)}] {
-	    set serial($name) 0
-	}
-	after 50 DeferWorkProc $name $serial($name)
-    }
-}
-proc DeferWorkCancel name {
-    upvar #0 $name queue
-    global serial
-
-    if [info exists queue] {
-	if [catch {incr serial($name)}] {
-	    set serial($name) 0
-	}
-	foreach w $queue {
-	    catch [lindex $w 1]
-	}
-	unset queue
-    }
-}
-proc DeferWorkProc {name incarnation} {
-    upvar #0 $name queue
-    global serial
-
-    if {$serial($name) != $incarnation} return
-
-    set this [lindex $queue 0]
-    set queue [lrange $queue 1 end]
-    catch [lindex $this 0]
-    if [llength $queue] {
-	after 20 DeferWorkProc $name $serial($name)
-    }
-}
-}
