@@ -31,15 +31,18 @@ proc Flag_Init {} {
     # that positions icons that can depend on the iconsize.
     Preferences_Resource flag(iconup) iconUpBitmap flagup.bitmap
     Preferences_Resource flag(icondown) iconDownBitmap flagdown.bitmap
-    Preferences_Resource flag(iconspool) iconSpoolBitmap flagup.bitmap
+    Preferences_Resource flag(iconspool) iconSpoolBitmap flagspool.bitmap
     Preferences_Resource flag(labelup) iconUpLabel {$flist(newMsgs) Unseen}
     Preferences_Resource flag(labeldown) iconDownLabel exmh
     Preferences_Resource flag(labelspool) iconSpoolLabel {$exmh(numUnInced) Spooled}
     Preferences_Resource flag(iconupmask) iconUpMask flagup.mask
     Preferences_Resource flag(icondownmask) iconDownMask flagdown.mask
     Preferences_Resource flag(iconspoolmask) iconSpoolMask flagspool.mask
+    Preferences_Resource flag(iconupglyph) iconUpGlyph flagup.gif
+    Preferences_Resource flag(icondownglyph) iconDownGlyph flagdown.gif
+    Preferences_Resource flag(iconspoolglyph) iconSpoolGlyph flagspool.gif
 
-    foreach i {iconup icondown iconspool iconupmask icondownmask iconspoolmask} {
+    foreach i {iconup icondown iconspool iconupmask icondownmask iconspoolmask iconupglyph icondownglyph iconspoolglyph} {
 	if ![string match /* $flag($i)] {
 	    set flag($i) $exmh(library)/$flag($i)
 	}
@@ -47,6 +50,23 @@ proc Flag_Init {} {
 	    set flag($i) 0
 	}
     }
+    if {!$exmh(slowDisp) || $exmh(slowDispIcons)} {
+	if [catch {
+	    Exmh_Debug "Creating .icon"
+	    toplevel .icon
+	    wm group .icon .
+	    pack [canvas .icon.c]
+	    image create photo icondown -file $flag(icondownglyph)
+	    image create photo iconup -file $flag(iconupglyph)
+	    image create photo iconspool -file $flag(iconspoolglyph)
+	    .icon.c configure -width [image width iconup] \
+		    -height [image height iconup]
+	    wm iconwindow . .icon
+	} err] {
+	    Exmh_Debug "Can't create .icon: $err"
+	    destroy .icon
+	}
+    }
     FlagInner down icondown labeldown
 }
 proc Flag_NewMail { {folder {}} } {
@@ -56,8 +76,11 @@ proc Flag_NewMail { {folder {}} } {
 # This is called after viewing a message
 proc Flag_MsgSeen { {folder {}} } {
     global flist
-    FlagInner down icondown \
-	[expr { ($flist(newMsgs) > 0) ? "labelup" : "labeldown" }]
+    if {$flist(newMsgs) > 0} {
+	FlagInner spool iconspool labelup
+    } else {
+	FlagInner down icondown labeldown
+    }
 }
 proc Flag_NoUnseen {} {
     FlagInner down icondown labeldown
@@ -70,12 +93,21 @@ proc Flag_NoSpooled {} {
 }
 proc FlagInner {state icon label} {
     global exmh flag
+    Exmh_Debug "In FlagInner $state $icon $label"
     if {$flag(state) != $state} {
-	wm iconbitmap . @$flag($icon)
-	if {$flag(${icon}mask) != 0} {
-	    wm iconmask . @$flag(${icon}mask)
+	if [winfo exists .icon.c] {
+	    Exmh_Debug "Setting flag glyph to $icon"
+	    .icon.c delete image -tag icon
+	    .icon.c create image 0 0 -anchor nw -image $icon -tag icon
+	} else {
+	    Exmh_Debug "Setting flag bitmap to $icon"
+	    wm iconbitmap . @$flag($icon)
+	    if {$flag(${icon}mask) != 0} {
+		wm iconmask . @$flag(${icon}mask)
+	    }
 	}
 	set flag(state) $state
+	Exmh_Debug "Set flag state to $state"
     }
     set l [uplevel #0 list $flag($label)]
     if {[info exists flag(lastLabel)] &&
