@@ -34,12 +34,12 @@ proc FlistResetVars {} {
     global flist
     set flist(unseen) {}	;# Sequence of folders to visit
     set flist(unvisited) {}	;# Unseen folders not yet visited
-    Exmh_Debug FlistInitVars
+    Exmh_Debug FlistResetVars
     set flist(unvisitedNext) {}	;# Temporary copy (next iteration)
     set flist(newMsgs) 0	;# Total count of unseen messages
-    set flist(newFolders) 0
 
     # flist(new,$folder)	is the number of unseen in a folder
+    #				(Update with care; 'trace'd by UnseenWinTrace)
     # flist(newseq,$folder)	is the message id's of unseen messages
     # flist(mtime,$folder)	is modify time of .mh_sequences file
     foreach x [array names flist] {
@@ -125,6 +125,7 @@ proc Flist_FindAllFolders {} {
 }
 proc FlistFindAllInner {} {
     global flist flistSubCache flistParents mhProfile
+    catch {destroy .scanning}
     Widget_Toplevel .scanning "Scanning..."
     Widget_Message .scanning msg -cursor watch -text "
 Scanning for nested folders.
@@ -287,17 +288,41 @@ proc FlistSeq { folder sequence } {
     return $seq
 }
 
+# Reset the cached state about unseen messages because the user
+# has just packed, sorted, or threaded the folder.
+# This should be followed shortly by a call to Flist_UnseenMsgs
+#
+# Don't call gratitiously because it confuses the exmhunseen window.
+
+proc Flist_ForgetUnseen {folder} {
+    global flist
+    set flist(newseq,$folder) {}
+    if {[info exists flist(new,$folder)]} {
+	set flist(newMsgs) [expr $flist(newMsgs) - $flist(new,$folder)]
+    }
+    set flist(new,$folder) 0
+    set ix [lsearch $flist(unseen) $folder]
+    if {$ix >= 0} {
+	set flist(unseen) [lreplace $flist(unseen) $ix $ix]
+    }
+}
+
 # Add unseen messages to the list for a given folder.
 # This has to be careful about already known unseen messages
 # and messages that have been read but not committed as read.
 proc Flist_AddUnseen {folder seq} {
     global flist exmh
-    Exmh_Debug Flist_AddUnseen 1 $folder $seq
 
     # Check overlap with already seen msgs and unseen messages already known
-    if ![info exists flist(newseq,$folder)] {
+    if [info exists flist(newseq,$folder)] {
+	if [info exists flist(new,$folder)] {
+	    set new $flist(new,$folder)
+	} else {
+	    set new 0
+	}
+    } else {
 	set flist(newseq,$folder) {}
-	set flist(new,$folder) 0
+	set new 0
     }
     if {[string compare $folder $exmh(folder)] == 0} {
 	set known [concat [Msg_Seen] $flist(newseq,$folder)]
@@ -326,11 +351,9 @@ proc Flist_AddUnseen {folder seq} {
 	return
     }
     incr flist(newMsgs) $num
-    incr flist(new,$folder) $num
+    set flist(new,$folder) [expr $new + $num]
     set flist(newseq,$folder) [concat $flist(newseq,$folder) $seq]
-    Exmh_Debug Flist_AddUnseen 2 $folder $flist(newseq,$folder)
     if {[lsearch $flist(unseen) $folder] < 0} {
-	incr flist(newFolders)
 	lappend flist(unseen) $folder
     }
     if {[string compare $folder $exmh(folder)] != 0 &&
@@ -338,6 +361,13 @@ proc Flist_AddUnseen {folder seq} {
 	lappend flist(unvisitedNext) $folder
     }
     Fdisp_HighlightUnseen $folder
+    if 0 {
+	# This line of code causes doubling of messages in the scan listing.
+	# don't do it
+	if {[string compare $exmh(folder) $folder] == 0} {
+	  Scan_Folder $folder
+	}
+    }
 }
 proc Flist_Done {} {
     global flist exmh
@@ -345,8 +375,8 @@ proc Flist_Done {} {
     Exmh_Debug Flist_Done
     if {$flist(newMsgs) > 0} {
 	if {$flist(newMsgs) == 1} {set msg "msg"} else {set msg "msgs"}
-	if {$flist(newFolders) == 1} {set f "folder"} else {set f "folders"}
-	Exmh_Status "$flist(newMsgs) unread $msg in $flist(newFolders) $f" blue
+	if {[llength $flist(unseen)] == 1} {set f "folder"} else {set f "folders"}
+	Exmh_Status "$flist(newMsgs) unread $msg in [llength $flist(unseen)] $f" blue
 	if ![info exists flist(lastNewMsgs)] {
 	    set flist(lastNewMsgs) 0
 	}
@@ -371,6 +401,7 @@ proc Flist_Done {} {
 
 proc Flist_UnseenUpdate { folder } {
     global exmh flist
+    Flist_ResetUnseen $folder
     Flist_UnseenMsgs $folder
     if {[string compare $folder $exmh(folder)] == 0} {
 	Scan_FolderUpdate $folder
@@ -382,6 +413,14 @@ proc Flist_UnseenUpdate { folder } {
     # This wiggles the flag and sorts flist(unvisited)
     Flist_Done
 }
+proc Flist_ResetUnseen { folder } {
+    global flist
+    if [Flist_NumUnseen $folder] {
+	incr flist(newMsgs) -$flist(new,$folder)
+	set flist(newseq,$folder) {}
+	set flist(new,$folder) 0
+    }
+}
 proc Flist_UnseenMsgs { folder } {
     global flist
     Flist_AddUnseen $folder [Mh_Unseen $folder]
@@ -410,21 +449,21 @@ proc FlistFindStart {reset} {
     if ![info exists flist(active)] {
 	set flist(active) 0
     }
-    Exmh_Debug FlistFindStart active $flist(active)
+    Exmh_Debug FlistFindStart reset=$reset active=$flist(active)
     if {$flist(active)} {
-	error "FlistActive"
+	return 0
     }
     set flist(active) 1
     if {$reset} {
 	Fdisp_ClearHighlights
 	FlistResetVars
     }
-    return 
+    return 1
 }
 proc FlistFindUnseen {reset} {
     global mhProfile flist curFolder
-    Exmh_Debug FlistFindUnseen reset $reset
-    if [catch {BgRPC FlistFindStart $reset}] {
+    Exmh_Debug FlistFindUnseen reset=$reset
+    if {![BgRPC FlistFindStart $reset]} {
 	# Flist active
 	return
     }
@@ -494,6 +533,9 @@ proc Flist_MsgSeen { msgid } {
 	}
     }
 }
+proc Flist_SeenAll { folder } {
+    FlistUnseenFolder $folder
+}
 proc FlistUnseenFolder { folder } {
     global flist
     Exmh_Debug FlistUnseenFolder $folder
@@ -503,7 +545,6 @@ proc FlistUnseenFolder { folder } {
     set ix [lsearch $flist(unseen) $folder]
     if {$ix >= 0} {
 	set flist(unseen) [lreplace $flist(unseen) $ix $ix]
-	incr flist(newFolders) -1
 	if {[llength $flist(unseen)] == 0} {
 	    Flag_NoUnseen
 	}
