@@ -15,7 +15,7 @@ proc Ftoc_Init {} {
     global ftoc
     set ftoc(displayValid) 0		;# 1 => pick results, not full scan
     set ftoc(displayDirty) 0		;# 1 => display differs from cache
-    set ftoc(mono) [string match mono* [tk colormodel .]]
+    set ftoc(mono) [expr {[winfo depth .] <= 4}]
     # Parameters to the Next button
     Preferences_Add "Scan Listing" \
 "These settings affect the behavior of Exmh as you move through the scan listing to view and mark messages.
@@ -94,12 +94,10 @@ proc Ftoc_Update { numMsgs folder } {
 
 proc Ftoc_Bindings { w } {
     # Bindings for the ftoc text widget
-    global tk_version
-    if {$tk_version >= 4.0} {
-	# The TScroll binding to too general.
-	# We'll do our own scroll bindings here.
-	bindtags $w [list $w]
-    }
+
+    # The TScroll binding to too general.
+    # We'll do our own scroll bindings here.
+    bindtags $w [list $w]
 
     # Button-1 starts selection range
     bind $w <Button-1> {
@@ -689,18 +687,18 @@ proc Ftoc_RescanLine { ix {plus none} } {
 	case $plus {
 	    "none" {
 		# Replace + (current marker) with blank
-		set ok [regsub {( *[0-9]+)(\+)} $text {\1 } newtext]
+		set ok [regsub {^( *[0-9]+)(\+)} $text {\1 } newtext]
 	    }
 	    "+" {
 		# Stick a + after the number, if needed
-		if ![regexp {( *)([0-9]+)(\+)} $text] {
-		    set ok [regsub {( *[0-9]+)( )} $text {\1+} newtext]
+		if ![regexp {^( *)([0-9]+)(\+)} $text] {
+		    set ok [regsub {^( *[0-9]+)( )} $text {\1+} newtext]
 		}
 	    }
 	    "dash" {
 		# Stick a - after the number, if needed
-		if ![regexp {( *)([0-9]+).-} $text] {
-		    set ok [regsub {( *[0-9]+.)(.)} $text {\1-} newtext]
+		if ![regexp {^( *)([0-9]+).-} $text] {
+		    set ok [regsub {^( *[0-9]+.)(.)} $text {\1-} newtext]
 		}
 		# Annotations result in writes to the directory.
 		# Here we mark the display dirty to force an update
