@@ -1,4 +1,3 @@
-# 
 # inc.tcl
 #
 # Incorporate new mail into folders.
@@ -30,11 +29,13 @@ proc Inc_Init {} {
     }
     Preferences_Add "Incorporate Mail" \
 "Exmh knows several ways to incorporate mail from your system's spool file into the MH folder hierarchy." {
-	{inc(style) incStyle	{CHOICE inbox presort multidrop none} {Ways to Inc}
+	{inc(style) incStyle	{CHOICE inbox presort multidrop presortmulti custom none} {Ways to Inc}
 "inbox - basic MH inc from your spool file to inbox folder.
 presort - slocal filtering directly into various folders.
 multidrop - slocal filtering or POP delivery into various drop boxes,
 as specified by ~/.xmhcheck, that are in turn inc'ed into folders.
+presortmulti - presort + multidrop.
+custom - use an Inc_Custom procedure, which is user supplied.
 none - you have an external agent that handles inc for you, so
 don't bother trying it from within exmh."}
 	{inc(onStartup) incOnStartup OFF	  {Inc at startup}
@@ -49,17 +50,21 @@ it's just sort of a heart-beat feature..."}
 	{inc(xnsgetmail) xnsGetMail OFF	  {Run xnsgetmail -k}
 "Run xnsgetmail -k before inc in order to fetch your
 XNS mail messages into your UNIX spool file."}
+	{inc(pophost) popHost {}	  {Mail host for POP3 protocol}
+"If this is set, inc will try to use the POP3 protocol to
+fetch your mail from a mail server."}
     }
 }
 proc Inc_Startup {} {
     global inc
+    if {! $inc(onStartup) || $inc(style) == "multidrop"} {
+	Exmh_Status "Checking folders"
+	Flist_FindUnseen
+    }
     if {$inc(onStartup)} {
 	set s [Sound_Off]
 	Inc
 	if {$s} { Sound_On }
-    } else {
-	Exmh_Status "Checking folders"
-	Flist_FindUnseen
     }
 }
 proc Inc_Mapped {} {
@@ -68,30 +73,74 @@ proc Inc_Mapped {} {
 	Inc
     }
 }
+
+# Inc_Show is appropriate for the Inc button, but not for the
+# background inc process.  It incs, changes to inbox, and shows
+# the first unseen message in the folder.  (That may or may not
+# be the first one just inc'ed.)
+
+proc Inc_Show {} {
+    global exmh
+
+    Inc
+    if { $exmh(folder) != "inbox" } {
+	Folder_Change inbox
+    }
+    Msg_ShowUnseen
+}
+
 proc Inc {} {
+    BgAction "Inc" IncInner	;# Execute in the background process.
+}
+proc IncInner {} {
     # There are three different styles of inc'ing messages...
     global inc
     if $inc(xnsgetmail) {
 	Xns_GetMail
     }
+    if {[string length $inc(pophost)]} {
+	# See if we know the password for this host
+	Pop_GetPassword $inc(pophost)
+    }
     case $inc(style) {
 	{default inbox}	{ busy Inc_Inbox }
 	presort		{ busy Inc_Presort }
 	multidrop	{ busy Inc_All }
+	presortmulti	{ busy Inc_PresortMultidrop }
+	custom		{ busy Inc_Custom }
 	none		{ return }
     }
 }
 
+proc Inc_PresortMultidrop {} {
+    # Copy from drop boxes to folders.  Anything
+    # that goes into MyIncTmp will get sorted later.
+    global mhProfile
+    if ![file isdirectory $mhProfile(path)/MyIncTmp] {
+	exec mkdir $mhProfile(path)/MyIncTmp
+    }
+    # Inc, but don't fiddle with scan listings
+    Inc_All 0
+
+    # Now do the Inc Presort on MyIncTmp and update scan listings
+    Inc_Presort
+}
+
 proc Inc_Inbox {} {
     # Inc from the spool file to the inbox folder
-    global exmh mhProfile ftoc
+    global exmh mhProfile ftoc pop
     if [info exists mhProfile(inbox)] {
 	set inbox [string trimleft $mhProfile(inbox) +]
     } else {
 	set inbox inbox
     }
-    if [catch {exec inc +$inbox -truncate -nochangecur -width $ftoc(scanWidth)} incout] {
-	Exmh_Debug Inc_Inbox +${inbox}: $incout
+    Exmh_Status "Inc ..."
+    set cmd [list exec inc +$inbox -truncate -nochangecur -width $ftoc(scanWidth)]
+    if {[info exist pop(password)]} {
+	lappend cmd << $pop(password)
+    }
+    if [catch $cmd incout] {
+	Exmh_Debug Inc_Inbox $cmd: $incout
 	set incout {}
     }
     BgRPC Inc_InboxFinish $inbox $incout Flist_Done
@@ -107,6 +156,7 @@ proc Inc_InboxFinish { f incout {hook {}} } {
     }
     Exmh_Debug Inc_InboxFinish $f $msgids hook=$hook
     if {[llength $msgids] == 0} {
+	Exmh_Status "No new messages in $f"
 	return
     }
     Flist_AddUnseen $f $msgids
@@ -116,19 +166,26 @@ proc Inc_InboxFinish { f incout {hook {}} } {
     if {$exmh(folder) == $f} {
 	Scan_Inc $f $incout
     } else {
-	Exmh_Status "New messages in $f" blue
+	Exmh_Status "New messages in $f"
     }
 }
 
-proc Inc_Presort {} {
+proc Inc_Presort {{doinc 1}} {
     # Transfer from the mail spool file directly into folders
-    global exmh mhProfile env inc
-    # Just use inc to copy stuff into a temp directory
-    catch {exec folder -push +[exec folder -fast] < /dev/null > /dev/null}
-    exec folder +MyIncTmp < /dev/null > /dev/null
-    if {[catch {exec inc +MyIncTmp -silent} err]} {
-	# Some versions of inc exit non-zero when they should not.
-	Exmh_Debug Inc_Presort +MyIncTmp: $err
+    global exmh mhProfile env inc pop
+    # Use inc to copy stuff into a temp directory
+    if ![file isdirectory $mhProfile(path)/MyIncTmp] {
+	exec mkdir $mhProfile(path)/MyIncTmp
+    }
+    if {$doinc} {
+	set cmd [list exec inc +MyIncTmp -silent]
+	if {[info exist pop(password)]} {
+	    lappend cmd << $pop(password)
+	}
+	if {[catch $cmd err]} {
+	    # Some versions of inc exit non-zero when they should not.
+	    Exmh_Debug Inc_Presort +MyIncTmp: $err
+	}
     }
     if [catch {set env(USER)} user] {
 	if [catch {set env(LOGNAME)} user] {
@@ -137,40 +194,60 @@ proc Inc_Presort {} {
 	}
     }
     if {[catch {glob $mhProfile(path)/MyIncTmp/*} files] == 0} {
-	Exmh_Status "presort inc ..." red
+	Exmh_Status "presort inc ..."
 	foreach file $files {
-	    if {$file == "++"} {
-		if [catch {exec rm $file} err] {
-		    Exmh_Debug Inc_Presort: $err
-		}
+	    if {[string compare [file tail $file] "++"] == 0} {
+		File_Delete $file
 		continue
 	    }
-	    # now use slocal to (presumably) shift things to a folder
-	    if [catch {exec $exmh(slocal) -user $user < $file}] {
-		Exmh_Status "Cannot classify $file"
-	    } else {
-		if [catch {exec rm $file} err] {
-		    Exmh_Debug Inc_Presort: $err
+	    if {$inc(presortFeedback)} {
+		if [catch {exec grep "^Subject:" $file | head -1} \
+		    subject] {
+		    set subject ""
 		}
+	    }
+		
+	    # now use slocal to (presumably) shift things to a folder
+	    if [catch {exec $exmh(slocal) -user $user < $file} err] {
+		# Move it out of MyIncTmp in case it really did
+		# get filed somewhere.  Certain file system errors
+		# (can't stat .) lead to this behavior
+		Exmh_Status "$file - $err - check MyIncErrors"
+		if ![file isdirectory $mhProfile(path)/MyIncErrors] {
+		    exec mkdir $mhProfile(path)/MyIncErrors
+		}
+		Mh_Refile MyIncTmp [file tail $file] MyIncErrors 
+	    } else {
+		File_Delete $file
 		if {$inc(presortFeedback)} {
-		    Exmh_Status [file tail $file] red
+		    if {[string length $subject] > 0} {
+			regsub "^Subject: *" $subject "" subject
+			Exmh_Status "[file tail $file]: $subject" red
+		    } else {
+			Exmh_Status [file tail $file] red
+		    }
 		}
 	    }
 	}
     }
-    if [catch {exec folder -pop < /dev/null > /dev/null} err] {
-	Exmh_Debug $err
-    }
-    catch {exec rm -f $mhProfile(path)/MyIncTmp/$mhProfile(mh-sequences)}
+    File_Delete $mhProfile(path)/MyIncTmp/$mhProfile(mh-sequences)
     catch {exec rmdir $mhProfile(path)/MyIncTmp}
 
     Flist_FindUnseen		;# Needed to set new message state.
-    BgRPC Inc_PresortFinish
+    # This after breaks a potential deadlock:
+    # UI Inc button is pressed - registers outstanding Inc operation
+    # PresortFinish notes new messages in the current folder and wants
+    # to rescan the folder to pick up the changes.
+    # The scan notes the outstanding operation and waits for it to complete.
+    # Deadlock
+    after 1 {BgRPC Inc_PresortFinish}
 }
 proc Inc_PresortFinish {} {
     global exmh ftoc
+    LOG Inc_PresortFinish
     Mh_Folder $exmh(folder)	;# prestort inc has changed this to MyIncTmp
-    if {[Flist_NumUnseen $exmh(folder)] > 0} {
+    if {$ftoc(displayValid) && [Flist_NumUnseen $exmh(folder)] > 0} {
+	Label_Folder $exmh(folder)
 	Scan_Folder $exmh(folder) $ftoc(showNew)
     }
 }
@@ -179,41 +256,52 @@ proc Inc_PresortFinish {} {
 # because you use maildelivery to divert messages into dropboxes.
 # This also works with POP hosts.
 
-proc Inc_All {} {
-    global exdrops exmh ftoc
+proc Inc_All {{updateScan 1}} {
+    global exdrops exmh ftoc pop
 
     Exmh_Status "Checking dropboxes..." red
     MhSetMailDrops	;# Update, if needed
+
     set hits {}
-    foreach folder [array names exdrops] {
-	set dropname $exdrops($folder)
+    foreach record [array names exdrops] {
+        set fields [ scan $exdrops($record) "%s %s %s" folder dropname popname]
+        Exmh_Status "dropbox... $dropname" red
 	if {[string first / $dropname] < 0} {
 	    # Not a pathname, but a POP hostname
 	    set host [lindex $dropname 0]
-	    Exmh_Status "$folder @ $host"
-	    if {[llength $dropname] == 2} {
-		set user [lindex $dropname 1]
-		set cmd [list exec inc +$folder -host $host -user $user -truncate -width $ftoc(scanWidth)]
+	    Exmh_Status "$popname @ $host"
+	    if {$popname != ""} {
+                set user $popname
+		set cmd [list exec inc +$folder -host $host \
+			-user $user -truncate -width $ftoc(scanWidth)]
 	    } else {
-		set cmd [list exec inc +$folder -host $host -truncate -width $ftoc(scanWidth)]
+		set cmd [list exec inc +$folder -host $host \
+			-truncate -width $ftoc(scanWidth)]
+	    }
+	    Pop_GetPassword $host
+	    if {[info exist pop($host,password)]} {
+		lappend cmd << $pop($host,password)
 	    }
 	} else {
 	    if { [file exists $dropname] && [file size $dropname] } {
-		lappend hits $folder
-		set cmd [list exec inc +$folder -file $dropname -truncate -width $ftoc(scanWidth)]
+		set cmd [list exec inc +$folder -file $dropname \
+			-truncate -width $ftoc(scanWidth)]
 	    } else {
 		set cmd {}
 	    }
 	}
 	if [llength $cmd] {
 	    if {[catch $cmd incout] == 0} {
-		BgRPC Inc_InboxFinish $folder $incout
+		lappend hits $folder
+	        if $updateScan {
+		    BgRPC Inc_InboxFinish $folder $incout
+		}
 	    } else {
 		Exmh_Debug Inc_All +${folder}: $incout
 	    }
 	}
     }
-    if {[llength $hits] > 0} {
+    if {$updateScan && ([llength $hits] > 0)} {
 	BgRPC Flist_Done
 	Exmh_Status "New mail in: $hits" blue
     } else {
@@ -224,8 +312,8 @@ proc Inc_Pending {} {
     # Figure out which folders have stuff in their inbox
     global exdrops exwin
     set active {}
-    foreach folder [array names exdrops] {
-	set dropfile $exdrops($folder)
+    foreach record [array names exdrops] {
+        set fields [ scan $exdrops($record) "%s %s %s" folder dropfile popname]
 	if {[file exists $dropfile] && [file size $dropfile]} {
 	    lappend active $folder
 	}
