@@ -14,11 +14,8 @@
 proc Exmh {} {
     global exmh argv nmh
 
-    if ![ catch {string match *-group* [exec repl -help] } ] {
-	set nmh 1
-    } else {
-	set nmh 0
-    }
+    set nmh 0
+    catch {string match *group* [exec repl -help] } nmh
 
     Mh_Init		;# Defines mhProfile
 
