@@ -99,6 +99,7 @@ proc MsgClear {} {
     $exwin(mtext) delete 0.0 end
     $exwin(mtext) configure -state disabled
     Face_Delete
+    catch {destroy $exwin(mopButtons).list}
 }
 proc Msg_ShowSomething {} {
     global exmh msg
@@ -141,6 +142,8 @@ proc MsgChange {msgid {show show} {line {}} } {
 	# Allow null msgid from Msg_ShowWhat, which supplies line instead
 	if {$msgid < 0}  return
     } else {
+        # line null too, try using first in folder
+        if {[string length $line] == 0} { set line 1 }
 	set msgid [Ftoc_MsgNumber [Ftoc_FindMsg $msgid $line]]
     }
     Ftoc_ClearCurrent
@@ -281,6 +284,17 @@ proc MsgComp {args} {
     }
     return 1
 }
+
+proc Msg_ReplyAll { } {
+    global nmh
+
+    if {$nmh == 1} {
+	Msg_Reply -group
+    } else {
+	Msg_Reply -cc to -cc cc
+    }
+}
+
 proc Msg_Reply { args } {
     global exmh msg mhProfile
     set exmh(ctype) {repl}
@@ -314,7 +328,7 @@ proc Msg_Reply { args } {
 		    }
 		}
 		eval {MhExec repl +$exmh(folder) $m -nowhatnowproc} $args
-		MhAnnoSetup $exmh(folder) $m repl
+		eval MhAnnoSetup $exmh(folder) $m repl $args
 	    } err] {	;# Setup draft msg
 		Exmh_Status "repl: $err" purple
 		Quote_Cleanup				;# Nuke @ link
@@ -330,7 +344,7 @@ proc Msg_Reply { args } {
 }
 
 proc Msg_Forward { args } {
-    global exmh msg
+    global exmh msg mhProfile
     set exmh(ctype) {forw}
     if {[string length $args] == 0} {
 	set args Mh_ForwSetup
@@ -371,7 +385,10 @@ proc Msg_Forward { args } {
 		    }
 		}
 		eval {MhExec forw +$exmh(folder)} $ids -nowhatnowproc $args
-		MhAnnoSetup $exmh(folder) $ids forw
+		eval MhAnnoSetup $exmh(folder) $ids forw $args
+		if {$mhProfile(forwtweak)} {
+		    Mh_Forw_MungeSubj $exmh(folder) $ids
+		}
 	    } err] {
 		Exmh_Status "forw: $err" purple
 		return
@@ -404,7 +421,7 @@ proc Msg_Dist { args } {
 	    if [catch {
 		Exmh_Status "dist +$exmh(folder) $m"
 		eval {MhExec dist +$exmh(folder) $m} -nowhatnowproc $args
-		MhAnnoSetup $exmh(folder) $m dist
+		eval MhAnnoSetup $exmh(folder) $m dist $args
 	    } err] {
 		Exmh_Status "dist: $err" purple
 		return
@@ -449,10 +466,10 @@ proc Msg_RemoveById { msgid {rmProc Ftoc_Delete} } {
     }
 }
 proc Msg_Move { {moveProc Ftoc_MoveMark} {advance 1} {show show} } {
-    global exmh
+    global exmh fdisp
 
     if {$exmh(target) == ""} {
-	Exmh_Status "Right click on folder label to pick destination" purple
+	Exmh_Status "Must first click button $fdisp(tarbutton) on folder label to pick destination" purple
 	return
     }
     if { $exmh(target) != $exmh(folder)} then {
@@ -465,7 +482,7 @@ proc Msg_Move { {moveProc Ftoc_MoveMark} {advance 1} {show show} } {
 	    Ftoc_NextImplied $show
 	}
     } else {
-	Exmh_Status "Move requires target folder != current folder"
+	Exmh_Status "Move or copy requires target folder != current folder"
     }
 }
 proc Msg_MoveNoshow { {moveProc Ftoc_MoveMark} } {
