@@ -34,7 +34,7 @@ proc Msg_Setup { t } {
 	}
     }
     # More tags to pre-allocate darker mime backgrounds.
-    if {[tk colormodel .] == "color"} {
+    if {[winfo depth .] > 4} {
 	set color default
 	foreach level {1 2 3 4 5 6} {
 	    set color [MimeDarkerColor $t $color]
@@ -81,7 +81,9 @@ proc MsgShow { msgid } {
 	set msg(curclear) 0
 	set mime(stop) 0
 	update idletasks	;# Faces display can be slow
-	Face_Show [MsgParseFrom $mimeHdr(0=1,hdr,from)] $mimeHdr(0=1,hdr,x-face)
+
+	Face_Show [MsgParseFrom $mimeHdr(0=1,hdr,from)] $mimeHdr(0=1,hdr,x-face) $mimeHdr(0=1,hdr,x-image-url) $mimeHdr(0=1,hdr,newsgroups)
+
 	foreach cmd [info commands Hook_MsgShow*] {
 	    $cmd $mhProfile(path)/$exmh(folder)/$msgid mimeHdr
 	}
@@ -109,6 +111,8 @@ proc MsgShowInText { win file } {
     set mimeHdr($part=$subpart,hdr,date) {}
     set mimeHdr($part=$subpart,hdr,subject) {}
     set mimeHdr($part=$subpart,hdr,x-face) {}
+    set mimeHdr($part=$subpart,hdr,x-image-url) {}
+    set mimeHdr($part=$subpart,hdr,newsgroups) {}
     set mimeHdr($part=$subpart,fullHeaders) $mime(fullHeaders)
     set mimeHdr($part=$subpart,yview) 1.0
 
@@ -181,3 +185,77 @@ proc MsgParseFrom { fromline {setaddr setaddr} } {
     return $token
 }
 
+proc Hook_MsgShowListHeaders {msgPath headervar} {
+    upvar $headervar header
+
+    global exwin
+    
+    # From rfc2369:
+  
+    # The contents of the list header fields mostly consist of angle-
+    # bracket ('<', '>') enclosed URLs, with internal whitespace being
+    # ignored. MTAs MUST NOT insert whitespace within the brackets, but
+    # client applications should treat any whitespace, that might be
+    # inserted by poorly behaved MTAs, as characters to ignore.
+    #
+    # A list of multiple, alternate, URLs MAY be specified by a comma-
+    # separated list of angle-bracket enclosed URLs. The URLs have order of
+    # preference from left to right. The client application should use the
+    # left most protocol that it supports, or knows how to access by a
+    # separate application. 
+    #
+    # [...]
+    #
+    # To allow for future extension, client applications MUST follow the
+    # following guidelines for handling the contents of the header fields
+    # described in this document:
+    #
+    # 1) Except where noted for specific fields, if the content of the
+    #    field (following any leading whitespace, including comments)
+    #    begins with any character other than the opening angle bracket
+    #    '<', the field SHOULD be ignored.
+    #
+    # 2) Any characters following an angle bracket enclosed URL SHOULD be
+    #    ignored, unless a comma is the first non-whitespace/comment
+    #    character after the closing angle bracket.
+    #
+    # 3) If a sub-item (comma-separated item) within the field is not an
+    #    angle-bracket enclosed URL, the remainder of the field (the
+    #    current, and all subsequent, sub-items) SHOULD be ignored.
+
+    # Loop through the list- headers
+    set menuitems {}
+    foreach index [array names header 0=1,hdr,list-*] {
+	# Get the suffix portion of the header name
+	regsub {^.*,list-} $index {} name
+	# Remove comments
+	regsub -all {\([^()]*\)} $header($index) {} h
+	# Remove whitespace
+	regsub -all " " $h {} h
+	# Loop through the fields
+	foreach f [split $h ,] {
+	    # Stricture #1
+	    if {[string index $f 0] == "<"} {
+		# Stricture #2
+		regexp "<(.*)>" $f match url
+		lappend menuitems $name $url
+	    } else {
+		# Stricture #3
+		break
+	    }
+	}
+    }
+    if {$menuitems != {}} {
+	if [winfo exists $exwin(mopButtons).list] {
+	    set menu $exwin(mopButtons).list.m
+	} else {
+	    set menu [Widget_AddMenuB $exwin(mopButtons) list "List..." {right padx 1 filly}]
+	}
+	$exwin(mopButtons).list.m delete 1 99
+	foreach {name url} $menuitems {
+	    Widget_AddMenuItem $menu $name [list URI_StartViewer $url]
+	}
+    } else {
+	catch {destroy $exwin(mopButtons).list}
+    }
+}
