@@ -6,6 +6,22 @@
 # 
 
 # $Log: pgpExec.tcl,v $
+# Revision 1.11  1999/09/27 23:18:45  kchrist
+# More PGP changes. Consolidated passphrase entry to sedit field or
+# pgpExec routine. Made the pgp-sedit field aware of pgp(keeppass)
+# and pgp(echopass). Moved pgp(keeppass), pgp(echopass) and
+# pgp(grabfocus) to PGP General Interface. Fixed a minor bug left
+# over from my previous GUI changes. Made pgp-sedit field appear and
+# disappear based on its enable preference setting.
+#
+# Revision 1.10  1999/09/22 16:36:44  kchrist
+# Changes made to support a different structure under the PGP Crypt... button.
+# Instead of an ON/OFF pgp($v,sign) variable now we use it to specify
+# the form of the signature (none, standard, detached, clear, or w/encrypt).
+# Code changed in several places to support this new variable definition.
+#
+# Updated Sedit.html to include a description of the new interface.
+#
 # Revision 1.9  1999/08/22 18:57:36  bmah
 # Sanitize PGP debugging entries before writing via Exmh_Debug.
 #
@@ -105,10 +121,10 @@ proc Pgp_Exec { v exectype arglist outvar {privatekey {}} {interactive 0} } {
     }
 
     set output {}
-    if {![set pgp($v,keeppass)]} {
+    if {![set pgp(keeppass)]} {
 	Pgp_ClearPassword $v
     }
-    if {$interactive || !([set pgp($v,keeppass)] || ($privatekey == {}))} {
+    if {$interactive || !([set pgp(keeppass)] || ($privatekey == {}))} {
         Exmh_Debug "<Pgp_Exec> Pgp_Exec_Interactive $v $exectype $arglist output"
 	return [Pgp_Exec_Interactive $v $exectype $arglist output]
     } else {
@@ -120,12 +136,8 @@ proc Pgp_Exec { v exectype arglist outvar {privatekey {}} {interactive 0} } {
 
 	    set keyid [lindex $privatekey 0]
 	    Exmh_Debug keyid=$keyid
-	    if {!$pgp(seditpgp)} {
-		Exmh_Debug "<Pgp_Exec> Pgp_GetPass $v $privatekey"
-		set p [Pgp_GetPass $v $privatekey]
-	    } else {
-		set p $pgp($v,pass,$keyid)
-	    }
+	    # Check for passphrase. Pgp_GetPass is cache and expire aware!
+	    set p [Pgp_GetPass $v $privatekey]
 	    #Exmh_Debug "<Pgp_Exec> got passwd >$p<"
 
 	    if {[string length $p] == 0} {
@@ -183,7 +195,7 @@ proc Pgp_Exec_Interactive { v exectype arglist outvar } {
     # Build shellcommand
     set shcmd "
         $pgpcmd \"[join [Pgp_Misc_Map x {
-	    regsub {([$"\`])} $x {\\1} x
+	    regsub {([$\"\`])} $x {\\1} x
 	    set dummy $x
         } $args] {" "}]\";
 	echo
@@ -382,30 +394,21 @@ proc Pgp_Exec_EncryptSign { v in out sigkey tokeys } {
     }
 }
  
-proc Pgp_Exec_Sign { v in out sigkey clear } {
+proc Pgp_Exec_Sign { v in out sigkey opt } {
     global pgp
 
-    Exmh_Debug "Pgp_Exec_Sign $v $in $out $sigkey $clear"
+    Exmh_Debug "Pgp_Exec_Sign $v $in $out $sigkey $opt"
 
     set keyid [lindex $sigkey 0]
-    if $clear {
-	Pgp_Exec $v sign [subst [set pgp($v,args_signClear)]] output $sigkey
-    } else {
-	Pgp_Exec $v sign [subst [set pgp($v,args_signBinary)]] output $sigkey
+    switch $opt {
+	standard {Pgp_Exec $v sign [subst [set pgp($v,args_signBinary)]] output $sigkey}
+	detached {Pgp_Exec $v sign [subst [set pgp($v,args_signDetached)]] output $sigkey}
+	clearsign {Pgp_Exec $v sign [subst [set pgp($v,args_signClear)]] output $sigkey}
+	default {set output "Pgp_Exec_Sign error. Unknown option."}
     }
     Pgp_Exec_CheckSuccess $v $out $output "signed text"
 }
     
-proc Pgp_Exec_SignDetached { v in out sigkey } {
-    global pgp
-
-    Exmh_Debug "Pgp_Exec_SignDetached $v $in $out $sigkey"
-
-    set keyid [lindex $sigkey 0]
-    Pgp_Exec $v sign [subst [set pgp($v,args_signDetached)]] output $sigkey
-    Pgp_Exec_CheckSuccess $v $out $output "signed text"
-}
-
 # Look if pgp generated pgp code
 proc Pgp_Exec_CheckSuccess {v out output object} {
     global pgp
@@ -442,10 +445,8 @@ proc Pgp_Exec_GetDecryptKey {v in recipients} {
     # and has set preferences to run pgp twice,
     # run pgp a first time to get out the decryption keyid
     set runtwice 0
-    if {[info exists pgp($v,runtwice)]} {
-      if {[set pgp($v,runtwice)]} {
+    if {[info exists pgp($v,runtwice)] && [set pgp($v,runtwice)]} {
         set runtwice 1
-      }
     }
     if {$runtwice} {
       Exmh_Debug "<Pgp_Exec_GetDecryptKey> Pgp_Exec_GetDecryptKeyid $v $in"
@@ -519,15 +520,6 @@ proc Pgp_Exec_Decrypt { v in out outvar recipients } {
     set key [Pgp_Exec_GetDecryptKey $v $in $recipients]
     Exmh_Debug "<Pgp_Exec_Decrypt> $key"
     
-    # See if we have a passphrase for this key.  If not, go get it.
-    set keyid [lindex $key 0]
-    if {![info exists pgp($v,pass,$keyid)]} {
-	set p [Pgp_GetPass $v $key]
-	if {[string length $p] == 0} {
-	    set pgp($v,pass,$keyid) $p
-	}
-    }
-
     Pgp_Exec $v verify [subst [set pgp($v,args_decrypt)]] output $key
 }
 
@@ -624,7 +616,8 @@ proc Pgp_Exec_ExtractKeys { v file outvar {interactive 1} } {
     }
 }
 
-# Get the passphrase for keyinstance key
+# Get the passphrase for keyinstance key. We also take care of setting
+# passphrase timeouts. Return a stored passphrase when possible.
 proc Pgp_GetPass { v key } {
     global pgp
 
@@ -634,16 +627,27 @@ proc Pgp_GetPass { v key } {
         return {}
     }
 
+    # Search the passphrase "cache". Need to set-timeout here in case
+    # the pass phrase was created via the seditpgp entry field.
     # Because of DecryptExpects asymmetric passphrase storage
     # we need to look for both mainkey and subkey separately
     set keyid [lindex $key 0]
     set subkeyid [lindex $key 2]
-    if {([info exists pgp($v,pass,$keyid)]) && ([string length $pgp($v,pass,$keyid)] > 0)} {
+    if {([info exists pgp($v,pass,$keyid)]) && \
+	    ([string length $pgp($v,pass,$keyid)] > 0)} {
+	Pgp_SetPassTimeout $v $keyid
+	if {[string length $subkeyid] > 0} {
+	    Pgp_SetPassTimeout $v $subkeyid
+	}
         return [set pgp($v,pass,$keyid)]
-    } elseif {([string length $subkeyid] > 0) && ([info exists pgp($v,pass,$subkeyid)]) && ([string length $pgp($v,pass,$subkeyid)] > 0)} {
+    } elseif {([string length $subkeyid] > 0) && \
+	    ([info exists pgp($v,pass,$subkeyid)]) && \
+	    ([string length $pgp($v,pass,$subkeyid)] > 0)} {
+	Pgp_SetPassTimeout $v $subkeyid
         return [set pgp($v,pass,$subkeyid)]
     }
 
+    # Not in "cache" (or expired) go ask for it.
     while 1 {
 	Exmh_Debug "Attempt to get passphrase for [lindex $key 0] [lindex $key 1] [lindex $key 4]"
         if [catch {Pgp_Misc_GetPass $v "Enter [set pgp($v,fullName)] passphrase" \
@@ -653,15 +657,15 @@ proc Pgp_GetPass { v key } {
             # SYMMETRIC ENCRYPTION
             return $password
         } elseif {[Pgp_Exec_CheckPassword $v $password $key]} {
-            if [set pgp($v,keeppass)] {
+            if [set pgp(keeppass)] {
                 set pgp($v,pass,$keyid) $password
+		Pgp_SetPassTimeout $v $keyid
                 # Because of DecryptExpect we need to store passphrase
                 # for mainkey and subkey
                 if {[string length $subkeyid] > 0} {
                     set pgp($v,pass,$subkeyid) $password
+		    Pgp_SetPassTimeout $v $subkeyid
                 }
-                after [expr [set pgp(passtimeout)] * 60 * 1000] \
-                        [list Pgp_ClearPassword $v $keyid]
             }
             return $password
         }
