@@ -1,6 +1,18 @@
 # pgpGpg.tcl
 
 # $Log: pgpGpg.tcl,v $
+# Revision 1.8  1999/09/30 03:51:07  kchrist
+# pgp($v,cmd_Beauty) was getting in the way of pgp($v,cmd_User) for
+# v=gpg so I had to rearrange things a bit.
+#
+# Revision 1.7  1999/09/27 23:18:45  kchrist
+# More PGP changes. Consolidated passphrase entry to sedit field or
+# pgpExec routine. Made the pgp-sedit field aware of pgp(keeppass)
+# and pgp(echopass). Moved pgp(keeppass), pgp(echopass) and
+# pgp(grabfocus) to PGP General Interface. Fixed a minor bug left
+# over from my previous GUI changes. Made pgp-sedit field appear and
+# disappear based on its enable preference setting.
+#
 # Revision 1.6  1999/08/24 15:51:07  bmah
 # Patch from Kevin Christian to make email PGP key queries work, and
 # to make key attachment RFC 2015 compliant.
@@ -61,7 +73,7 @@ Give here a hkp server name." }
 
 # Needed for Preferences
 set pgp(gpg,description) "GNUPG is a free GPLed PGP clone written by Werner Koch"
-set pgp(gpg,prefs) [list keeppass echopass grabfocus rfc822 \
+set pgp(gpg,prefs) [list rfc822 \
                          choosekey runtwice cacheids minmatch showinline \
                          shortmsgs autoextract \
                          keyserver keyquerymethod HKPkeyserverUrl keyserverUrl \
@@ -423,12 +435,9 @@ set pgp(gpg,pat_GoodSignature) {GOODSIG}
 set pgp(gpg,pat_Untrusted) {(TRUST_UNDEFINED|TRUST_NEVER)}
 set pgp(gpg,pat_BadSignature) {BADSIG}
 set pgp(gpg,pat_UnknownError) {ERROR}
-
-################
-# ShortenOutput
 # command that matches out the Originator
 set pgp(gpg,cmd_User) {
-    regexp {(GOODSIG|BADSIG) [^ ]* ([^ \n]*)} $pgpresult {} {} user
+    regexp {(GOODSIG|BADSIG) [^ ]* ([^\n]*)} $in {} {} user
 }
 
 ##################
