@@ -19,6 +19,35 @@
 # to avoid auto-loading this whole file.
 
 # $Log: pgpMain.tcl,v $
+# Revision 1.17  1999/10/07 14:29:28  kchrist
+# Changed quote file name documentation string to remove ambiguity regarding
+# the location of this file.
+#
+# Fixed a PGP bug when sign-encrypting a MIME message.
+#
+# Revision 1.16  1999/09/30 14:55:20  kchrist
+# One more fix to the cmd_User issue.
+#
+# Revision 1.15  1999/09/30 03:51:07  kchrist
+# pgp($v,cmd_Beauty) was getting in the way of pgp($v,cmd_User) for
+# v=gpg so I had to rearrange things a bit.
+#
+# Revision 1.14  1999/09/27 23:18:45  kchrist
+# More PGP changes. Consolidated passphrase entry to sedit field or
+# pgpExec routine. Made the pgp-sedit field aware of pgp(keeppass)
+# and pgp(echopass). Moved pgp(keeppass), pgp(echopass) and
+# pgp(grabfocus) to PGP General Interface. Fixed a minor bug left
+# over from my previous GUI changes. Made pgp-sedit field appear and
+# disappear based on its enable preference setting.
+#
+# Revision 1.13  1999/09/22 16:36:44  kchrist
+# Changes made to support a different structure under the PGP Crypt... button.
+# Instead of an ON/OFF pgp($v,sign) variable now we use it to specify
+# the form of the signature (none, standard, detached, clear, or w/encrypt).
+# Code changed in several places to support this new variable definition.
+#
+# Updated Sedit.html to include a description of the new interface.
+#
 # Revision 1.12  1999/08/24 15:51:07  bmah
 # Patch from Kevin Christian to make email PGP key queries work, and
 # to make key attachment RFC 2015 compliant.
@@ -310,10 +339,10 @@ proc Pgp_ExmhEncrypt { v } {
 #    set pgp(param,recipients) [lindex $pgp($v,myname) 0]
     set id [SeditId $file]
     set pgp(encrypt,$id) 1;
-    set pgp(sign,$id) 0;
+    set pgp(sign,$id) "none";
     set pgp(format,$id) "pm";
 
-    Pgp_Process $v $file $tmpfile 1
+    Pgp_Process $v $file $tmpfile
 
     Mh_Rename $tmpfile $file
 
@@ -323,6 +352,7 @@ proc Pgp_ExmhEncrypt { v } {
     return 1
 }
 
+# XXX Orphaned code (pgp-action headers no longer used)
 # Removes any pgp-action header and inserts a brand new one
 proc Pgp_SeditEncrypt { action v draft t } {
     global pgp
@@ -399,12 +429,12 @@ proc Pgp_EncryptDebug { srcfile } {
     }
     
     # if there is nothing to do, stop here
-    if {!$pgp(encrypt,$id) && !$pgp(sign,$id)} {
+    if {!$pgp(encrypt,$id) && $pgp(sign,$id)=="none"} {
 	Exmh_Debug "Pgp_EncryptDebug: No action"
     }
 
     # check originator (if necessary)
-    if $pgp(sign,$id) {
+    if {$pgp(sign,$id) != "none"} {
 	if [info exists pgp(param,originator)] {
 	    set originator [PgpMatch_Simple $pgp(param,originator) $pgp(secring)]
 	} else {
@@ -434,7 +464,8 @@ proc Pgp_EncryptDebug { srcfile } {
 
 }
 
-# Choose the private key to use
+# Choose the private key to use. Default is 1st element of pgp($v,myname) 
+# which is set by Pgp_Exec_Init.
 proc Pgp_ChoosePrivateKey { v text } {
     global pgp
 
@@ -448,27 +479,34 @@ proc Pgp_ChoosePrivateKey { v text } {
     }
 }
 
+# Change pgp($v,myname,$id) variable and update pgp(cur,pass,$id) along 
+# the way. Function is called from the Crypt... menu in Sedit and WhatNow.
 proc Pgp_SetMyName { v id } {
    global pgp
 
-# first, save old pgp passphrase if set
-   if {[info exists pgp($v,pass,cur$id)] && [info exists pgp($v,myname,$id)]} {
+    # first, save old pgp passphrase if set
+   if {[info exists pgp(cur,pass,$id)] && \
+	   ([string length $pgp(cur,pass,$id)] > 0) && \
+	   [info exists pgp($v,myname,$id)]} {
       set keyid [lindex $pgp($v,myname,$id) 0]
-      set pgp($v,pass,$keyid) $pgp($v,pass,cur$id)
+      set pgp($v,pass,$keyid) $pgp(cur,pass,$id)
+      if {![info exists pgp(timeout,$keyid)]} {
+	 Pgp_SetPassTimeout $v $keyid
+      }
    }
 
    set newname [Pgp_ChoosePrivateKey $v \
 	 "Please select a $pgp($v,fullName) key to use for signing"]
-   if ([string length $newname]) {
+   if {[string length $newname]} {
        set pgp($v,myname,$id) [lindex $newname 0]
 
        Exmh_Debug "Pgp_SetMyName: myname now $pgp($v,myname,$id)"
 
        set keyid [lindex $pgp($v,myname,$id) 0]
        if [info exists pgp($v,pass,$keyid)] {
-	   set pgp($v,pass,cur$id) $pgp($v,pass,$keyid)
+	   set pgp(cur,pass,$id) $pgp($v,pass,$keyid)
        } else {
-	   set pgp($v,pass,cur$id) {}
+	   set pgp(cur,pass,$id) {}
        }
        Pgp_SetSeditPgpName $pgp($v,myname,$id) $id
    }
@@ -492,11 +530,16 @@ proc Pgp_SetSeditPgpName { myname id } {
 proc Pgp_SetSeditPgpVersion { v id } {
     global pgp
 
-    # Get old PGP passphrase and save it away if it exists.
+    # Get old PGP passphrase and save it away if it has content.
     set oldv $pgp(version,$id)
-    if {[info exists pgp(cur,pass,$id)] && [info exists pgp($oldv,myname,$id)]} {
+    if {[info exists pgp(cur,pass,$id)] && \
+	    ([string length $pgp(cur,pass,$id)] > 0) && \
+	    [info exists pgp($oldv,myname,$id)]} {
 	set keyid [lindex $pgp($oldv,myname,$id) 0]
 	set pgp($oldv,pass,$keyid) $pgp(cur,pass,$id)
+	if {![info exists pgp(timeout,$keyid)]} {
+	    Pgp_SetPassTimeout $oldv $keyid
+	}
     }
 
     # If we didn't pick a key for this PGP version yet in this
@@ -518,7 +561,7 @@ proc Pgp_SetSeditPgpVersion { v id } {
 
 }
 
-proc Pgp_Process { v srcfile dstfile {pgpaction {}} } {
+proc Pgp_Process { v srcfile dstfile } {
     global pgp env miscRE
 
     set orig [open $srcfile r]
@@ -538,7 +581,7 @@ proc Pgp_Process { v srcfile dstfile {pgpaction {}} } {
     }
     
     # if there is nothing to do, stop here
-    if {!$pgp(encrypt,$id) && !$pgp(sign,$id)} {
+    if {!$pgp(encrypt,$id) && $pgp(sign,$id)=="none"} {
 	close $orig
 	error "no action"
     }
@@ -546,16 +589,13 @@ proc Pgp_Process { v srcfile dstfile {pgpaction {}} } {
     if {$pgp(format,$id) == "app"} {
 	Exmh_Debug app format
 	if {$pgp(encrypt,$id)} {
-	    if {$pgp(sign,$id)} {
-		set typeparams "; x-action=encryptsign;"
-	    } else {
-		set typeparams "; x-action=encrypt;"
-	    }
+	    set typeparams "; x-action=encrypt;"
 	} else {
-	    if {$pgp(clearsign,$id)} {
-		set typeparams "; x-action=signclear;"
-	    } else {
-		set typeparams "; x-action=signbinary;"
+	    switch $pgp(sign,$id) {
+		detached -
+		standard {set typeparams "; x-action=signbinary;"}
+		clearsign {set typeparams "; x-action=signclear;"}
+		encryptsign {set typeparams "; x-action=encryptsign;"}
 	    }
 	}
     }
@@ -565,10 +605,10 @@ proc Pgp_Process { v srcfile dstfile {pgpaction {}} } {
 	set rfc822 [regexp -nocase $miscRE(true) $pgp(param,rfc822)]
     } else {
 	set rfc822 $pgp($v,rfc822)
-  }
+    }
 
     # setup the originator (if necessary)
-    if $pgp(sign,$id) {
+    if {$pgp(sign,$id) != "none"} {
 	Exmh_Debug PGP signing
 	if [info exists pgp(param,originator)] {
 	    set originator [PgpMatch_Simple $pgp(param,originator) $pgp(secring)]
@@ -581,7 +621,7 @@ proc Pgp_Process { v srcfile dstfile {pgpaction {}} } {
     }
 
     # get the ids of the recipients (if necessary)
-    if $pgp(encrypt,$id) {
+    if {$pgp(encrypt,$id) || $pgp(sign,$id) == "encryptsign"} {
 	Exmh_Debug PGP encrypting
 	if [info exists pgp(param,recipients)] {
 	    set ids [Pgp_Misc_Map id {PgpMatch_Simple $id $pgp($v,pubring)} \
@@ -595,7 +635,7 @@ proc Pgp_Process { v srcfile dstfile {pgpaction {}} } {
 	if {$pgp(format,$id) == "app"} {
 	    append typeparams ";\n\tx-recipients=\"[join [Pgp_Misc_Map key {string range [lindex $key 0] 2 end} $ids] ", "]\""
 	}
-      }
+    }
       
     # remove pgp-action and mime-version headers
     set mailheaders [Pgp_Misc_Filter line \
@@ -615,8 +655,8 @@ proc Pgp_Process { v srcfile dstfile {pgpaction {}} } {
     # write the message to be encrypted
     set msgfile [Mime_TempFile "msg"]
     set msg [open $msgfile w 0600]
-    if {$pgp(format,$id) == "plain"} {
-	set pgp(mime,$id) 0
+    if {$pgp(format,$id) == "plain" && $pgp(sign,$id) == "detached"} {
+	set pgp(sign,$id) "standard"
     } else {
 	foreach line $pgpheaders { puts $msg [Pgp_Misc_FixHeader $line] }
     }
@@ -627,22 +667,17 @@ proc Pgp_Process { v srcfile dstfile {pgpaction {}} } {
 
     set pgpfile [Mime_TempFile "pgp"]
     if [catch {
-	Exmh_Debug "encrypt=$pgp(encrypt,$id); sign=$pgp(sign,$id); mime=$pgp(mime,$id); clearsign=$pgp(clearsign,$id)"
+	Exmh_Debug "encrypt=$pgp(encrypt,$id); sign=$pgp(sign,$id)"
 	if {$pgp(encrypt,$id)} {
-	    if {$pgp(sign,$id)} {
-		Pgp_Exec_EncryptSign $pgp(version,$id) $msgfile $pgpfile $originator $ids 
-	    } else {
-		Pgp_Exec_Encrypt $pgp(version,$id) $msgfile $pgpfile $ids 
-	    }
+	    Pgp_Exec_Encrypt $pgp(version,$id) $msgfile $pgpfile $ids 
 	} else {
-	    if {$pgp(sign,$id)} {
-		if {$pgp(mime,$id)} {
-		    Pgp_Exec_SignDetached $pgp(version,$id) $msgfile $pgpfile $originator 
-		} else { 
-		    Pgp_Exec_Sign $pgp(version,$id) $msgfile $pgpfile $originator $pgp(clearsign,$id) 
-		}
-	    } else {
-		error "<PGP> Message is neither signed, nor encrypted"
+	    switch $pgp(sign,$id) {
+		standard -
+		detached -
+		clearsign {Pgp_Exec_Sign $pgp(version,$id) $msgfile $pgpfile $originator $pgp(sign,$id)}
+		encryptsign {Pgp_Exec_EncryptSign $pgp(version,$id) $msgfile $pgpfile $originator $ids}
+		none -
+		default {error "<PGP> Message is neither signed, nor encrypted"}
 	    }
 	}
     } err] {
@@ -668,7 +703,6 @@ proc Pgp_Process { v srcfile dstfile {pgpaction {}} } {
         }
     }
     
-    
     File_Delete $msgfile $pgpfile
 }
 
@@ -699,7 +733,7 @@ proc Pgp_ProcessPM {v dstfile pgpfile mailheaders plainfile id} {
 
     # Put in specified headers.  
     lappend mailheaders "mime-version: 1.0"
-    if {$pgp(encrypt,$id)} {
+    if {$pgp(encrypt,$id)  || $pgp(sign,$id) == "encryptsign"} {
 	lappend mailheaders "content-type: multipart/encrypted; boundary=\"$boundary\";\n\t protocol=\"application/pgp-encrypted\""
     } else {
 	lappend mailheaders "content-type: multipart/signed; boundary=\"$boundary\";\n\t micalg=pgp-${micalg}; protocol=\"application/pgp-signature\""
@@ -713,7 +747,7 @@ proc Pgp_ProcessPM {v dstfile pgpfile mailheaders plainfile id} {
     foreach line $mailheaders { puts $dst [Pgp_Misc_FixHeader $line] }
     puts $dst ""
     puts $dst "--$boundary"
-    if {$pgp(encrypt,$id)} {
+    if {$pgp(encrypt,$id) || $pgp(sign,$id) == "encryptsign"} {
 	puts $dst "Content-Type: application/pgp-encrypted"
 	puts $dst ""
 	puts $dst "Version: 1"
@@ -953,7 +987,7 @@ proc Pgp_MimeShowMultipartEncryptedPgp {tkw part} {
     # Decide whether or not to use expect
     set decrypt 1
     if {[info exists pgp($v,useexpectk)]} {
-        if {[set pgp($v,keeppass)] && \
+        if {[set pgp(keeppass)] && \
                     [info exists exmh(expectk)] && [set pgp($v,useexpectk)]} {
 	    # Decrypt with expect
 	    Pgp_Exec_DecryptExpect $v $mimeHdr($part=2,file) $tmpfile msg
@@ -1163,7 +1197,7 @@ proc Pgp_ShowMessage { tkw part } {
 		# Decide whether or not to use expect
                 set decrypt 1
                 if {[info exists pgp($v,useexpectk)]} {
-		    if {[set pgp($v,keeppass)] && [info exists exmh(expectk)] \
+		    if {[set pgp(keeppass)] && [info exists exmh(expectk)] \
 			&& [set pgp($v,useexpectk)]} {
 		        Exmh_Debug "<Pgp_ShowMessage> Using expect"
 		        Pgp_Exec_DecryptExpect $v $mimeHdr($part,file) $tmpfile msg
@@ -1373,6 +1407,9 @@ proc Pgp_InterpretOutput { v in outvar } {
     Exmh_Debug "<Pgp_InterpretOutput> PGP Output:\n$in"
     regexp {(.*)child process exited abnormally} $in {} in
     set in [string trim $in]
+    if {[string length $in] == 0} {
+	set in "PGP execution produced no messages."
+    }
 
     set pgpresult(ok) 1
 
@@ -1406,6 +1443,12 @@ proc Pgp_InterpretOutput { v in outvar } {
         set redin $in
     }
 
+    # get out user (for ShortenOutput) now since beautify erases needed info
+    eval [set pgp($v,cmd_User)]
+    if {![info exists user]} {
+	set user UNKNOWN
+    }
+
     Exmh_Debug <TUNING>
     # An output tuning command
     # NOTE: pgpresult(msg) should also be set there
@@ -1430,16 +1473,13 @@ proc Pgp_InterpretOutput { v in outvar } {
 
     if [set pgp($v,shortmsgs)] {
 	set pgpresult(msg) [Pgp_ShortenOutput $v $pgpresult(msg) \
-				$pgpresult(summary)]
+				$pgpresult(summary) $user]
     }
 }
 
-proc Pgp_ShortenOutput { v pgpresult summary } {
-
+proc Pgp_ShortenOutput { v pgpresult summary user } {
     global pgp
 
-    # get out user
-    eval [set pgp($v,cmd_User)]
     catch {Exmh_Debug "<PGP ShortenOutput> $user"}
 
     switch $summary {
