@@ -102,25 +102,30 @@ proc ScanFolder {F adjustDisplay} {
     Ftoc_ShowUnseen $F
     return
 }
-proc Scan_FolderForce {} {
+proc Scan_FolderForce {{F ""}} {
     global exmh mhProfile ftoc
-    set F $exmh(folder)
+    if {$F == ""} {
+	set F $exmh(folder)
+    }
     set cacheFile $mhProfile(path)/$F/.xmhcache
     if {$F == ""} {
 	Exmh_Status "No current folder" red
-    } else {
-	if {! [Ftoc_Changes Rescan]} {
-	    Background_Wait
-	    Label_Folder $F
-	    Exmh_Status "rescanning $F ..."
-	    Scan_IO $F [open "|$mhProfile(scan-proc) +$F -width $ftoc(scanWidth)"]
-	    set ftoc(displayValid) 1
-	    set ftoc(displayDirty) 1
-	    Ftoc_Yview end
-	    Flist_ForgetUnseen $F
-	    Ftoc_ShowUnseen $F
-	    Exmh_Status ok
-	}
+    } elseif {$F != $exmh(folder)} {
+	# If we aren't currently viewing the folder, just delete
+	# the cache file and we'll take care of this later
+	Exmh_Debug "Clearing $cacheFile"
+	file delete $cacheFile
+    } elseif {! [Ftoc_Changes Rescan]} {
+	Background_Wait
+	Label_Folder $F
+	Exmh_Status "rescanning $F ..."
+	Scan_IO $F [open "|$mhProfile(scan-proc) +$F -width $ftoc(scanWidth)"]
+	set ftoc(displayValid) 1
+	set ftoc(displayDirty) 1
+	Ftoc_Yview end
+	Flist_ForgetUnseen $F
+	Ftoc_ShowUnseen $F
+	Exmh_Status ok
     }
 }
 proc Scan_FolderUpdate { f } {
@@ -272,12 +277,7 @@ proc Scan_CacheUpdate {} {
     } elseif [catch {
 	set cacheIO [open $cacheFile w]
 	set curLine [Ftoc_ClearCurrent]			;# Clear +
-	global tk_version
-	if {$tk_version < 4.0} {
-	    set display [$exwin(ftext) get 1.0 end]
-	} else {
-	    set display [$exwin(ftext) get 1.0 "end -1 char"]
-	}
+	set display [$exwin(ftext) get 1.0 "end -1 char"]
 	Ftoc_Change [Ftoc_MsgNumber $curLine] $curLine	;# Restore it
 	puts $cacheIO $display nonewline
 	close $cacheIO
