@@ -25,7 +25,7 @@ proc Sedit_BindInit {} {
     # with the use of resources
     set sedit(defaultfile) $exmh(library)/sedit-bindings
     set sedit(localfile)   $exmh(library)/local.sedit-bindings
-    set sedit(dotfile)     ~/.exmhsedit
+    set sedit(dotfile)     ~/.exmh/exmhsedit
 
     SeditReadPref
     Sedit_ClassBindings
@@ -41,11 +41,6 @@ proc SeditReadPref {} {
 	    }
 	}
     }
-    # Hack out the <Button-2> binding from selpaste.
-    set ix [lsearch $sedit(key,selpaste) <Button-2>]
-    if {$ix >= 0} {
-	set sedit(key,selpaste) [lreplace $sedit(key,selpaste) $ix $ix]
-    }
 }
 proc SeditBind { class key body } {
     global sedit
@@ -61,19 +56,14 @@ proc SeditBind { class key body } {
 	    }
 	    # Make leading keystroke harmless
 	    if [regexp {(<.+>)<.+>} $seq match prefix] {
-		global tk_version
-		if {$tk_version >= 4.0} {
-		    bind $class $prefix break
-		} else {
-		    bind $class $prefix { }
-		}
+		bind $class $prefix break
 	    }
 	}
     } err] {
 	if ![info exists sedit(key,$key)] {
-	    puts stderr "Bind $class $key: $err"
+	    Exmh_Debug "Bind $class $key: $err"
 	} else {
-	    puts stderr "Bind $class $key $sedit(key,$key): $err"
+	    Exmh_Debug "Bind $class $key $sedit(key,$key): $err"
 	}
     }
 }
@@ -85,44 +75,43 @@ proc Sedit_TagBindings { w tag } {
     $w tag bind $tag <Any-ButtonRelease-1>	{WidgetTextSelDone %W}
 }
 proc SeditTextBindings { draft t } {
-    global sedit tk_version
-    if {$tk_version >= 4.0} {
-	# Define binding tags:
-	# SeditText - simple editor commands and data entry
-	# TScroll - drag scrolling
-	# TSelect - text selection
-	# $t - send message binding
-	# toplevel - not used
-	# all - not used because it has focus-change bindings on <Tab>
-	bindtags $t [list $t SeditText TSelect TScroll [winfo toplevel $t]]
-	SeditBind $t sendMsg "SeditSend $draft $t ; break"
-    } else {
-	SeditBind $t sendMsg [list SeditSend $draft $t]
-    }
+    global sedit quote
+    # Define binding tags:
+    # SeditText - simple editor commands and data entry
+    # TScroll - drag scrolling
+    # TSelect - text selection
+    # $t - send message binding
+    # toplevel - not used
+    # all - not used because it has focus-change bindings on <Tab>
+    bindtags $t [list $t SeditText TSelect TScroll [winfo toplevel $t]]
+    SeditBind $t sendMsg "SeditSend $draft $t 0 ; break"
     SeditBind Entry sendMsg { }
+    SeditBind $t abortMsg "SeditAbort $draft $t ; break"
+    SeditBind Entry abortMsg { }
+    SeditBind $t quoteMsg "SeditInsertFile $draft $t $quote(filename) ; break"
+    SeditBind Entry quoteMsg { }
+
+    # redisplay debug loggin
+    if {[info commands log_dump] == "log_dump"} {
+	bind $t <Key> {log "Key %A %K"}
+	bind $t <Enter> {log "Enter %W"}
+	bind $t <Leave> {log "Leave %W"}
+	bind $t <ButtonPress> {log "ButtonPress %W"}
+	bind $t <ButtonRelease> {log "ButtonRelease %W"}
+    }
 }
 proc Sedit_ClassBindings { } {
-    global sedit tk_version
+    global sedit
 
-    # Reset everything
-    if {$tk_version < 4.0} {
-	foreach class {Text Entry} {
-	    foreach b [bind $class] {
-		bind $class $b ""
-	    }
+    foreach class {SeditText Entry} {
+	foreach b [bind $class] {
+	    bind $class $b ""
 	}
-	set tclass Text
-    } else {
-	foreach class {SeditText Entry} {
-	    foreach b [bind $class] {
-		bind $class $b ""
-	    }
-	}
-	# This is needed because there are no Text bindings at this
-	# point - they have not been faulted in yet from the library -
-	# So erasing them now doesn't help.  We use an alternate class.
-	set tclass SeditText
     }
+    # This is needed because there are no Text bindings at this
+    # point - they have not been faulted in yet from the library -
+    # So erasing them now doesn't help.  We use an alternate class.
+    set tclass SeditText
 
     # Modification bindings
 
@@ -171,42 +160,44 @@ proc Sedit_ClassBindings { } {
     }
 
 
-    # when the compose char is pressed, wait for 2 chars
-    # (the first may be part of the composechar event)
-    if {$tk_version >= 4.0} {
-	SeditBind $tclass composechar {
-	    bind SeditText <Any-Key> {
-		if {"%%A" != "{}"} {
-		    if {"%A" == "{}"} {
-			bind SeditText <Any-Key> {
-			    if {"%%%%A" != "{}"} {
-				SeditComposedKey %%%%W "%%A" "%%%%A"
-			    }
-			}
-		    } else {
-			SeditComposedKey %%W "%A" "%%A"
-		    }
-		}
+    # When <Dcircumflex_accent> is pressed, wait for one char and insert the
+    # compose key ^ <letter>
+    SeditBind $tclass <Dcircumflex_accent> {
+	bind SeditText <Any-Key> {
+	    if {"%%A" != "{}"} {
+		SeditComposedKey %%W "^" "%%A"
 	    }
 	}
-    } else {
-	SeditBind Text composechar {
-	    bind Text <Any-Key> {
-		if {"%%A" != {}} {
-		    if {"%A" == {}} {
-			bind Text <Any-Key> {
-			    if {"%%%%A" != {}} {
-				SeditComposedKey %%%%W "%%A" "%%%%A"
-			    }
+    }
+
+    # When <Ddiaeresis> is pressed, wait for one char and insert the
+    # compose key " <letter>
+    SeditBind $tclass <Ddiaeresis> {
+	bind SeditText <Any-Key> {
+	    if {"%%A" != "{}"} {
+		SeditComposedKey %%W "\"" "%%A"
+	    }
+	}
+    }
+
+    # when the compose char is pressed, wait for 2 chars
+    # (the first may be part of the composechar event)
+    SeditBind $tclass composechar {
+	bind SeditText <Any-Key> {
+	    if {"%%A" != "{}"} {
+		if {"%A" == "{}"} {
+		    bind SeditText <Any-Key> {
+			if {"%%%%A" != "{}"} {
+			    SeditComposedKey %%%%W "%%A" "%%%%A"
 			}
-		    } else {
-			SeditComposedKey %%W "%A" "%%A"
 		    }
+		} else {
+		    SeditComposedKey %%W "%A" "%%A"
 		}
 	    }
 	}
     }
-    
+
     SeditBind $tclass seldelete {
 	Text_KillSelection %W
 	SeditDirty %W
@@ -223,7 +214,7 @@ proc Sedit_ClassBindings { } {
 	SeditDirty %W
     }
     SeditBind Entry backspace {
-	tk_entryBackspace %W
+	tkEntryBackspace %W
     }
 
     SeditBind $tclass openline {
@@ -279,7 +270,9 @@ proc Sedit_ClassBindings { } {
 	SeditDirty %W
     }
     SeditBind Entry delwordback {
-	tk_entryBackword %W; tk_entrySeeCaret %W
+	%W delete [string wordstart [%W get] [expr [%W index insert] - 1]] \
+		insert
+	tkEntrySeeInsert %W
     }
 
     SeditBind $tclass delchar {
@@ -331,7 +324,7 @@ proc Sedit_ClassBindings { } {
     }
     SeditBind Entry linestart {
 	%W icursor 0
-	tk_entrySeeCaret %W
+	tkEntrySeeInsert %W
    }
 
     SeditBind $tclass lineend {
@@ -339,7 +332,7 @@ proc Sedit_ClassBindings { } {
     }
     SeditBind Entry lineend {
 	%W icursor end
-	tk_entrySeeCaret %W
+	tkEntrySeeInsert %W
     }
 
     set sedit(lastpos,Text) {}
@@ -368,7 +361,7 @@ proc Sedit_ClassBindings { } {
 	    }
 	}
 	%W icursor $x
-	tk_entrySeeCaret %W
+	tkEntrySeeInsert %W
     }
 
     SeditBind $tclass forwword {
@@ -387,7 +380,7 @@ proc Sedit_ClassBindings { } {
 	    }
 	}
 	%W icursor $x	
-	tk_entrySeeCaret %W
+	tkEntrySeeInsert %W
     }
 
     SeditBind $tclass backchar {
@@ -398,7 +391,7 @@ proc Sedit_ClassBindings { } {
 	if {$x > 0} {
 	    incr x -1
 	    %W icursor $x
-	    tk_entrySeeCaret %W
+	    tkEntrySeeInsert %W
 	}
     }
 
@@ -409,7 +402,7 @@ proc Sedit_ClassBindings { } {
 	set x [%W index insert]
 	incr x
 	%W icursor $x
-	tk_entrySeeCaret %W
+	tkEntrySeeInsert %W
     }
 
     SeditBind $tclass up1page {
@@ -431,14 +424,15 @@ proc Sedit_ClassBindings { } {
 	bind $tclass <Control-Shift_R> {SeditKinput_start %W}
     }
 
+    SeditBind SeditText addrexpand {Addr_KeyExpand %W}
+    SeditBind SeditText format {Sedit_FormatParagraph %W}
+
     foreach cmd [info commands Hook_TextBind*] {
 	$cmd $tclass
     }
 
     # Selection bindings
-    if {$tk_version >= 4.0} {
-	set tclass TSelect
-    }
+    set tclass TSelect
     SeditBind $tclass selcopy {
 	catch {clipboard clear ; clipboard append [Text_Selection]}
 	cutbuffer set [Text_Selection]
@@ -448,10 +442,51 @@ proc Sedit_ClassBindings { } {
 	cutbuffer set [Text_Selection]
     }
 
-    bind $tclass <Button-1>	{WidgetTextSelBegin %W %x %y char}
+    SeditBind $tclass selclear { Text_LoseSelection %W }
+    SeditBind Entry selclear { %W select clear }
+
+    SeditBind $tclass selstart {
+	Text_LoseSelection %W; WidgetTextSelStart %W insert char
+    }
+    SeditBind Entry selstart {
+	%W select clear ; %W select from insert
+    }
+
+    SeditBind $tclass selforw {
+	Text_SelectTo %W insert
+	Text_SelectionEnd %W 1
+    }
+    SeditBind Entry selforw {
+	set x [%W index insert]
+	incr x
+	%W icursor $x
+	%W select to insert
+    }
+
+    SeditBind $tclass selback {
+	Text_SelectTo %W insert
+	Text_SelectionEnd %W 1
+    }
+    SeditBind Entry selback {
+	set x [%W index insert]
+	incr x -1
+	%W icursor $x
+	%W select to insert
+    }
+
+    SeditBind $tclass selextend {
+	Text_SelectTo %W insert
+	Text_SelectionEnd %W 1
+	Text_SelectionEnd %W 1
+    }
+    SeditBind Entry selextend {
+	%W select to insert
+    }
+
+    bind $tclass <Button-1>		{WidgetTextSelBegin %W %x %y char}
     bind $tclass <Double-Button-1>	{WidgetTextSelBegin %W %x %y word}
     bind $tclass <Triple-Button-1>	{WidgetTextSelBegin %W %x %y line}
-    bind $tclass <B1-Motion>	{WidgetTextSelMotion %W %x %y}
+    bind $tclass <B1-Motion>		{WidgetTextSelMotion %W %x %y}
     bind $tclass <ButtonRelease-1>	{WidgetTextSelDone %W}
     bind $tclass <Shift-Button-1>	{WidgetTextSelAgain %W %x %y}
     bind $tclass <Shift-B1-Motion>	{WidgetTextSelMotion %W %x %y}
@@ -473,9 +508,7 @@ proc Sedit_ClassBindings { } {
     bind Entry <Control-u>	{%W delete 0 end}
 
     # Scroll bindings
-    if {$tk_version >= 4.0} {
-	set tclass TScroll
-    }
+    set tclass TScroll
 
     # Clear default scroll bindings
     foreach seq {<Button-2> <B2-Motion>} {
@@ -490,6 +523,25 @@ proc Sedit_ClassBindings { } {
 	ShiftMiddle { set b shift2 }
 	None { set b {} }
     }
+    # focus debug hack
+    if {([info command log_dump] == "log_dump")} {
+	bind $tclass <Double-Escape> {
+	    if [catch {
+		if ![info exists logID] {set logID 0}
+		while {[file exists /tmp/log$logID]} {
+		    incr logID
+		}
+		log SEDIT "/tmp/log$logID"
+		set out [open /tmp/log$logID w]
+		puts $out [log_dump]
+		close $out
+		Exmh_Status "Debug log in /tmp/log$logID"
+		incr logID
+	    } err] {
+		Exmh_Status $err
+	    }
+	}
+    }
     if {$b == 2 || $b == 3} {
 	bind $tclass <Button-$b> {
 	    WidgetTextMark %W %y
@@ -568,7 +620,7 @@ proc SeditEntryInsert { w a } {
 	    catch {$w delete sel.first sel.last}
 	}
 	$w insert insert $a
-	tk_entrySeeCaret $w
+	tkEntrySeeInsert $w
     }
 }
 proc SeditEntrySelect { w x } {
@@ -586,7 +638,7 @@ proc SeditEntryShiftSelect { w x } {
     }
     if {$sedit(selectmode,$w) == "word"} {
 	set nx [$w index @$x]
-	set ix [SeditEntryFindWord $w $x [expr {$nx >= $sedit(anchor,$w)}]
+	set ix [SeditEntryFindWord $w $x [expr {$nx >= $sedit(anchor,$w)}]]
     } else {
 	set ix [$w index @$x]
     }
@@ -611,7 +663,7 @@ proc SeditEntryWordSelect { w x } {
     set ix [SeditEntryFindWord $w $x 0]
     $w select from $ix
     $w icursor $ix
-    tk_entrySeeCaret $w
+    tkEntrySeeInsert $w
     $w select to [SeditEntryFindWord $w $x 1]
     set sedit(anchor,$w) $ix
 }
@@ -648,7 +700,7 @@ proc SeditEntryLineSelect { w x } {
     $w select from 0
     $w select to end
     $w icursor 0
-    tk_entrySeeCaret $w
+    tkEntrySeeInsert $w
 }
 proc SeditEntryTranspose { w {how ""} } {
     set _x [$w index insert]
@@ -669,7 +721,7 @@ proc SeditEntryTranspose { w {how ""} } {
 	incr _x
 	$w insert $_x $_c
     }
-    tk_entrySeeCaret $w
+    tkEntrySeeInsert $w
 }
 
 proc SeditKill?Sel { w } {
@@ -691,11 +743,43 @@ proc SeditInsert { w a } {
 	if {$X > 127} {
 	    set sedit($w,8bit) 1
 	}
-	$w insert insert $a
+	if {[string compare $sedit($w,format) "OnType"] == 0} {
+	    set i [SeditCount $w insert]
+	    if {$i >= $sedit(lineLength)} {
+		if [$w compare insert <= hlimit] {
+		    set stuff "\n "
+		} else {
+		    set stuff \n
+		}
+		if [catch {$w insert wordbreak $stuff}] {
+		    $w insert insert $stuff
+		}
+		$w mark unset wordbreak
+	    }
+	    $w insert insert $a
+	    if [regexp $sedit(wordbreakpat) $a] {
+		$w mark set wordbreak insert
+		$w mark gravity wordbreak left
+	    }
+	} else {
+	    $w insert insert $a
+	}
 	$w yview -pickplace insert
 	set sedit($w,dirty) 1
     }
 }
+proc SeditCount {w mark} {
+    set string [$w get "$mark linestart" $mark]
+    set i 0
+    foreach c [split $string {}] {
+	if {"$c" == "\t"} {
+	    set i [expr $i/8 + 8]
+	} else {
+	    incr i
+	}
+    }
+    return $i
+}
 proc SeditMarkClean { t } {
     global sedit
     set sedit($t,dirty) 0
@@ -762,7 +846,6 @@ proc Sedit_Pref {} {
 	    }
 	}
     }
-    Exwin_ToplevelFocus .seditpref none
 }
 proc SeditPrefItem { frame width name keystroke } {
     global sedit
@@ -812,20 +895,10 @@ proc SeditKinput_start { w } {
 }
 
 proc SeditEventHelp {} {
-    global tk_version
     set t .seditevent.text
     if [Exwin_Toplevel .seditevent "Event Helper" Pref] {
 	Widget_SimpleText .seditevent text {expand fill} -width 40 -height 10
-	if {$tk_version >= 4.0} {
-	    bind $t <Key> {SeditEventFeedback %W %K %A ; break}
-	} else {
-	    foreach seq [bind $t] {
-		if [string match Key* $seq] {
-		    bind $t $seq {}
-		}
-	    }
-	    bind $t <Any-Key> {SeditEventFeedback %W %K %A}
-	}
+	bind $t <Key> {SeditEventFeedback %W %K %A ; break}
     }
     $t delete 1.0 end
     $t insert 1.0 "Type into this window to see what events\nare generated by your keyboard\n***\n"
@@ -847,12 +920,3 @@ proc SeditEventFeedback {t keysym char} {
     }
     $t yview -pickplace end
 }
-
-if {$tk_version >= 4.0} {
-    proc tk_entrySeeCaret { w } { tkEntrySeeInsert $w }
-    proc tk_entryBackspace { w } { tkEntryBackspace $w }
-    proc tk_entryBackword { w } {
-	$w delete [string wordstart [$w get] [expr [$w index insert] - 1]] \
-		insert
-    }
-}
