@@ -13,7 +13,7 @@
 #
 
 proc SeditWhom { draft f t } {
-    global tk_version sedit
+    global sedit
     set parent [file root $f]
     if {[winfo exists $parent.whom]} {
 	destroy $parent.whom
@@ -41,11 +41,7 @@ proc SeditWhom { draft f t } {
     $t2 configure -height $height	;# Widget_Text broken
     $t2 insert 1.0 $result
     $t2 config -state disabled
-    if {$tk_version >= 3.3} {
-	pack $f2 -before $f -side top
-    } else {
-	pack before $f $f2 top
-    }
+    pack $f2 -before $f -side top
 }
 proc SeditSign { draft t {f ~/.signature} } {
     global sedit
@@ -416,6 +412,7 @@ proc SeditInsertFileDialog { draft t } {
 	if [file readable $name] {
 	    set options [SeditFormatDialog $t $name]
 	    eval {SeditInsertFile $draft $t $name} $options
+	    Sedit_FixPgpFormat [SeditId $draft]
 	} else {
 	    SeditMsg $t "Cannot read $name"
 	}
@@ -438,6 +435,7 @@ proc SeditInsertExternalDialog { draft t } {
 					[exec uname -n]]
     close $fp
     eval {SeditInsertFile $draft $t $tmpfname} $options
+    Sedit_PgpFormat [SeditId $draft]
     File_Delete $tmpfname
 }
 proc SeditExternalDialog { t name } {
@@ -577,7 +575,7 @@ proc SeditFormatNewPart { t f {doit 0} } {
     destroy $f
 }
 proc SeditSpell { draft f t } {
-    global tk_version sedit editor wish
+    global sedit editor wish
     set parent [file root $f]
     if {[winfo exists $parent.spell]} {
 	destroy $parent.spell
@@ -624,11 +622,7 @@ proc SeditSpell { draft f t } {
     $t2 configure -height $height	;# Widget_Text broken
     $t2 insert 1.0 $result
     $t2 config -state disabled
-    if {$tk_version >= 3.3} {
-	pack $f2 -before $f -side top
-    } else {
-	pack before $f $f2 top
-    }
+    pack $f2 -before $f -side top
 }
 proc Sedit_Find {draft t} {
     global sedit
@@ -669,27 +663,27 @@ proc SeditGuessContentType { filenameOrig } {
 
     set filename [string tolower $filenameOrig]
     set type {}
-    if [catch {set type [mailcap_guess_content_type $filename]} ] {
-	if ![info exists mimeType] {
-	    SeditLoadMimeTypes
+    if ![info exists mimeType] {
+	SeditLoadMimeTypes
+    }
+    if [regexp -- {^([1-9][0-9]*|@)$} [file tail $filename]] {
+	return message/rfc822
+    }
+    set suffix [file extension $filename]
+    set newfilename [file rootname $filename]
+    while {$newfilename != $filename} {
+	if [info exists mimeType($suffix)] {
+	    set type $mimeType($suffix)
 	}
-	if [regexp -- {^([1-9][0-9]*|@)$} [file tail $filename]] {
-	    return message/rfc822
-	}
-	set suffix [file extension $filename]
+	set filename $newfilename
+	set suffix "[file extension $filename]$suffix"
 	set newfilename [file rootname $filename]
-	while {$newfilename != $filename} {
-	    if [info exists mimeType($suffix)] {
-		set type $mimeType($suffix)
-	    }
-	    set filename $newfilename
-	    set suffix "[file extension $filename]$suffix"
-	    set newfilename [file rootname $filename]
-	}
     }
     if {[string length $type] == 0} {
-	if {[string length [set type [Mime_Magic $filenameOrig]]] == 0} {
-	    return $sedit(defaultType)
+	if [catch {set type [mailcap_guess_content_type $filename]}] {
+	    if {[string length [set type [Mime_Magic $filenameOrig]]] == 0} {
+		return $sedit(defaultType)
+	    }
 	}
     }
     return $type
@@ -809,12 +803,52 @@ proc SeditClip {draft t} {
     Msg_Clip $f $id
 }
 
+proc DecodeURL { url } {
+    regsub -all -nocase {%0a} $url {}   url
+    regsub -all -nocase {%0d} $url "\n" url
+    regsub -all         {%20} $url { }  url
+    regsub -all         {%25} $url {%}  url
+    regsub -all -nocase {%2c} $url {,}  url
+    regsub -all -nocase {%3c} $url {<}  url
+    regsub -all -nocase {%3d} $url {=}  url
+    regsub -all -nocase {%3e} $url {>}  url
+    regsub -all -nocase {%3f} $url {?}  url
+    return $url
+}
+
 proc Sedit_Mailto { url } {
     global mhProfile sedit
+
     MhExec comp -nowhatnowproc
     Sedit_Start [Mh_Path $mhProfile(draft-folder) cur]
+    # See RFC 2368 for mailto: URL syntax
     regsub mailto: $url {} url
-    SeditSetHeader $sedit(t) to $url 
+    if {[regexp -nocase {\?} $url]} {
+      regsub -nocase {.*\?} $url {} headers
+      foreach hdr [split $headers &] {
+        if {[regexp -nocase {body=} $hdr]} {
+          regsub -nocase {body=} $hdr {} body
+          set body [DecodeURL $body]
+          $sedit(t) insert "header + 1 line" "\n$body"
+        } else {
+          regsub {=.*} $hdr {} hdr_name
+          set hdr_name [string tolower $hdr_name]
+          regsub {.*=} $hdr {} hdr_value
+          set hdr_value [DecodeURL $hdr_value]
+          if {[string compare $hdr_name to] == 0} {
+            set to $hdr_value
+          }
+          SeditSetHeader $sedit(t) $hdr_name $hdr_value 
+        }
+      }
+      regsub -nocase {\?.*} $url {} url
+    }
+    set url [DecodeURL $url]
+    if [info exists to] {
+      SeditSetHeader $sedit(t) to "$url, $to" 
+    } else {
+      SeditSetHeader $sedit(t) to $url
+    }
 }
 # Run MHN now to format a message
 proc SeditMHN {draft t} {
@@ -873,26 +907,38 @@ proc SeditExternalCmd { draft t cmd } {
 }
 
 proc SeditAttachQuotedMessage { draft t name } {
-	global sedit
-	if {$name != ""} {
-		if [file readable $name] {
-			set options [SeditFormatDialog $t $name]
-			eval {SeditInsertFile $draft $t $name} $options
-		} else {
-			SeditMsg $t "Cannot read $name"
-		}
+    global sedit
+    if {$name != ""} {
+	if [file readable $name] {
+	    set options [SeditFormatDialog $t $name]
+	    eval {SeditInsertFile $draft $t $name} $options
+	} else {
+	    SeditMsg $t "Cannot read $name"
 	}
+    }
 }
 
 proc SeditInsertMessageDialog { draft t } {
-	global sedit msg
-	set name [FSBox "Select message" $msg(path)]
-	if {$name != ""} {
-		if [file readable $name] {
-		set options [SeditFormatDialog $t $name]
-		eval {SeditInsertFile $draft $t $name} $options
-		} else {
-			SeditMsg $t "Cannot read $name"
-		}
+    global sedit msg
+    set name [FSBox "Select message" $msg(path)]
+    if {$name != ""} {
+	if [file readable $name] {
+	    set options [SeditFormatDialog $t $name]
+	    eval {SeditInsertFile $draft $t $name} $options
+	    Sedit_FixPgpFormat [SeditId $draft]
+	} else {
+	    SeditMsg $t "Cannot read $name"
 	}
+    }
 }
+
+# I had this in pgp.tcl, but I don't want that entire file loaded
+# if this is called.
+proc Sedit_FixPgpFormat {id} {
+    global pgp
+    if {$pgp(enabled) && ($pgp(format,$id) == "plain")} {
+	Exmh_Status "Changed PGP encoding from plain to multipart"
+	set pgp(format,$id) "pm"
+    }
+}
+
