@@ -85,9 +85,10 @@ proc SeditMimeParse { t } {
 			    set sedit($t,level,$level) 0
 			}
 			$t mark set level=$level $i.0
+			$t mark set multi${level}.next end
 		    }
 		    if {[string match text/* $type]} {
-			$t tag add Charset $i.0 "$i.0 lineend"
+			$t tag add Charset beginpart "beginpart lineend"
 		    }
 		    foreach sub [lrange $params 1 end] {
 			if [regexp {([^=]+)=(.+)} $sub match key val] {
@@ -103,6 +104,9 @@ proc SeditMimeParse { t } {
 		}
 	    }
 	} else {
+	    if [regexp "\[\x80-\xff\]" $line] {
+		set sedit($t,8bit) 1
+	    }
 	    foreach b $boundaries {
 		if [regexp ^--$b\(--\)?\$ $line match alldone] {
 		    if {![string match multipart/* $type]} {
@@ -189,7 +193,13 @@ proc SeditMimeType { type {promote {}} } {
 	    regexp {part=(.+)} $tag match part
 	}
 	if {$promote && [string match {text/*} $type]} {
-	    set sedit($t,newpart) 0	;# change type
+	    if {$part == ""} {
+		# lost part headers? 
+		set sedit($t,newpart) 1
+	    } else {
+		# change existing part
+		set sedit($t,newpart) 0
+	    }
 	} else {
 	    SeditPartDialog $t $oldtype $type $level $part
 	}
@@ -222,11 +232,12 @@ proc SeditMimeFirstPart { t type promote keep} {
 
     # Position the header mark at the begining of the line where
     # we will be inserting new headers (Content-Type, etc.)
-
+    if [catch {$t index header}] {
+	SeditMsg $t "No message?"
+	$t mark set header end
+    }
     $t insert header "\nMime-Version: 1.0"
-    global tk_version
-    if {(($tk_version >= 4.0) && [$t compare header == "end -1c"]) ||
-	(($tk_version < 4.0) && [$t compare header == "end"])} {
+    if [$t compare header == "end -1c"] {
 	# Nothing left after deleting the body
 	$t insert header "\n"
     } else {
@@ -340,6 +351,7 @@ proc SeditStartMulti {t type {empty {}} } {
     global sedit
     set sedit($t,multi) 1
     set sedit($t,level,0) 0
+    set sedit($t,mhn) 0			;# Don't try MHN
 
     Exmh_Debug SeditStartMulti
 
@@ -388,9 +400,8 @@ proc SeditMultiInner {t type level mark mark2} {
     # Insert the terminating --boundary-- line and set multiN.next to be
     # just before that --boundary-- line.
 
-    global tk_version
     $t insert $mark2 \n
-    if {$tk_version >= 4.0 && ([string compare $mark2 end] == 0)} {
+    if {[string compare $mark2 end] == 0} {
 	set save [$t index "end -1 line"]
     } else {
 	set save [$t index $mark2]
@@ -413,8 +424,7 @@ proc SeditMultiInner {t type level mark mark2} {
 proc SeditBoundary {t level} {
     global sedit
     if ![info exists sedit($t,boundary,$level)] {
-	regsub -all "\[ \x7f-\xff\]" [exec date] _ date
-	set sedit($t,boundary,$level) ===_${level}_${date}
+	set sedit($t,boundary,$level) [Mime_MakeBoundary $level]
     }
     return $sedit($t,boundary,$level)
 }
@@ -428,6 +438,9 @@ proc SeditBoundary&Type {t type level {mark {}} {mark2 {}} } {
     if {[string length $mark] == 0} {
 	set mark multi${level}.next
     }
+    if [catch {$t index $mark}] {
+	$t mark set $mark end
+    }
     set boundary [SeditBoundary $t $level]
     scan [$t index $mark] "%d.%d" line char
     if {$char != 0} {
@@ -468,29 +481,36 @@ proc SeditContentType { t type level mark {mark2 {}} } {
 	set mark addpart
     }
     set save [$t index $mark]
-    if {$sedit(colorize) && ([tk colormodel .] == "color")} {
+    if {$sedit(colorize) && ([winfo depth .] > 4)} {
 	catch {
 	    switch -glob -- $type {
 		text/enriched* {
-		    $t tag configure type=$type -background pink
+		    $t tag configure type=$type -background $sedit(c_enrichedBg)
+		    $t tag configure type=$type -foreground $sedit(c_enrichedFg)
 		}
 		text/* {
-		    $t tag configure type=$type -background snow
+		    $t tag configure type=$type -background $sedit(c_textBg)
+		    $t tag configure type=$type -foreground $sedit(c_textFg)
 		}
 		audio/* {
-		    $t tag configure type=$type -background gold
+		    $t tag configure type=$type -background $sedit(c_audioBg)
+		    $t tag configure type=$type -foreground $sedit(c_audioFg)
 		}
 		image/* {
-		    $t tag configure type=$type -background powderblue
+		    $t tag configure type=$type -background $sedit(c_imageBg)
+		    $t tag configure type=$type -foreground $sedit(c_imageFg)
 		}
 		message/* {
-		    $t tag configure type=$type -background seashell
+		    $t tag configure type=$type -background $sedit(c_messageBg)
+		    $t tag configure type=$type -foreground $sedit(c_messageFg)
 		}
 		application/* {
-		    $t tag configure type=$type -background honeydew
+		    $t tag configure type=$type -background $sedit(c_applicationBg)
+		    $t tag configure type=$type -foreground $sedit(c_applicationFg)
 		}
 		video/* {
-		    $t tag configure type=$type -background lavenderblush
+		    $t tag configure type=$type -background $sedit(c_videoBg)
+		    $t tag configure type=$type -foreground $sedit(c_videoFg)
 		}
 	    }
 	}
@@ -572,7 +592,7 @@ after the current part #${part}?"
     tkwait window $f
     return $sedit($t,newpart)
 }
-proc SeditFormatMail { t out } {
+proc SeditFormatMail { t out isigw } {
     global sedit exmh
     set tag [$t tag names]
     Exmh_Debug FormatMail tags $tag
@@ -588,6 +608,30 @@ proc SeditFormatMail { t out } {
 	set ranges {}
 	set quote 0
     }
+    set id $sedit($t,id)
+    SeditCheckForIsigHeaders $t
+    if {$sedit($t,format) == "OnType"} {
+	# Prevent duplicate X-Mailer or X-Exmh-Isig-* headers
+	if {[catch {set end [$t index hlimit]}] &&
+	    [catch {set end [$t index header]}]} {
+		set end end
+	}
+	set X1 [$t get 1.0 $end]
+	set X2 [$t get $end end]
+	regsub -all -nocase "(^|\n)(x-mailer:\[^\n\]*\n)+" $X1 {\1} X1
+	regsub -all -nocase "(^|\n)(x-exmh-isig-(comptype|folder):\[^\n\]*\n)+" $X1 {\1} X1
+	# No X-Mailer on redistributed messages
+	if {[string compare $exmh($id,action) dist] != 0} {
+	    puts $out "X-Mailer: exmh $exmh(version)"
+	}
+	# Replace X-Exmh-Isig-* headers if necessary
+	if {$isigw} {
+	    puts $out "X-Exmh-Isig-CompType: $sedit($t,isigc)"
+	    puts $out "X-Exmh-Isig-Folder: $sedit($t,isigf)"
+	}
+	puts $out "$X1$X2"
+	return
+    }
     # Try not to butcher non-text parts
     set breakrange {}
     foreach tg $tag {
@@ -618,10 +662,14 @@ proc SeditFormatMail { t out } {
     set plen [string length $sedit(pref,replPrefix)]
 
     # No X-Mailer on redistributed messages
-    set id $sedit($t,id)
     if {[string compare $exmh($id,action) dist] != 0} {
 	puts $out "X-Mailer: exmh $exmh(version)"
     }
+    # Output X-Exmh-Isig-* headers if necessary
+    if {$isigw} {
+	puts $out "X-Exmh-Isig-CompType: $sedit($t,isigc)"
+	puts $out "X-Exmh-Isig-Folder: $sedit($t,isigf)"
+    }
     for {set L 1} {$L <= $last} {incr L} {
 	set line [$t get $L.0 $L.end]
 	if {$inheaders} {
@@ -630,7 +678,8 @@ proc SeditFormatMail { t out } {
 	    if {[regexp {^[ 	]*$} $line] || [regexp {^--+} $line]} {
 		set inheaders 0
 	    }
-	    if {[regexp -nocase {^x-mailer:} $line]} {
+	    if {[regexp -nocase {^x-mailer:} $line] ||
+		[regexp -nocase {^x-exmh-isig-(comptype|folder):} $line]} {
 		continue
 	    }
 	}
@@ -708,7 +757,7 @@ proc SeditFormatMail { t out } {
 		puts $out \ $newline
 	    }
 	    incr c
-	    set line [string range $line $c end]
+	    set line [string trimright [string range $line $c end]]
 	    if {$inheaders} {
 		set continuation 1
 		set limit $climit
@@ -778,6 +827,7 @@ proc SeditTestInsert { file {newpart 1} {encoding {}} {type text/plain} {desc {}
     global sedit
     set t $sedit(t)
     SeditInsertFile {} $t $file $newpart
+    Sedit_FixPgpFormat [SeditId $file]
 }
 proc SeditTestInit { {charset 0} } {
     global sedit
